from typing_extensions import Literal, NotRequired, TypedDict

import discord_typings

__all__ = (
    'EntitlementData',
    'EntitlementTypes',
)


# https://discord.com/developers/docs/monetization/entitlements#entitlement-object


class EntitlementData(TypedDict):
    id: 'discord_typings.Snowflake'
    sku_id: 'discord_typings.Snowflake'
    application_id: 'discord_typings.Snowflake'
    user_id: NotRequired['discord_typings.Snowflake']
    type: 'discord_typings.EntitlementTypes'
    deleted: bool
    starts_at: NotRequired[str]
    ends_at: NotRequired[str]
    guild_id: NotRequired['discord_typings.Snowflake']


# https://discord.com/developers/docs/monetization/entitlements#entitlement-object-entitlement-types


EntitlementTypes = Literal[8]



from typing import Dict, List, Optional, Union

from typing_extensions import Literal, NotRequired, TypedDict

import discord_typings

__all__ = (
    'PartialChannelData',
    'TextChannelData',
    'NewsChannelData',
    'DMChannelData',
    'GroupDMChannelData',
    'ThreadChannelData',
    'VoiceChannelData',
    'CategoryChannelData',
    'ForumChannelData',
    'MediaChannelData',
    'ChannelData',
    'ChannelTypes',
    'VideoQualityModes',
    'SortOrderTypes',
    'ForumLayoutTypes',
    'MessageData',
    'UserMentionData',
    'MessageTypes',
    'MessageActivityData',
    'MessageActivityTypes',
    'MessageInteractionMetadataData',
    'MessageReferenceData',
    'FollowedChannelData',
    'MessageReactionData',
    'ReactionCountDetailsData',
    'PermissionOverwriteData',
    'ThreadMetadataData',
    'ThreadMemberData',
    'DefaultReactionData',
    'ForumTagData',
    'EmbedData',
    'EmbedThumbnailData',
    'EmbedVideoData',
    'EmbedImageData',
    'EmbedProviderData',
    'EmbedAuthorData',
    'EmbedFooterData',
    'EmbedFieldData',
    'PartialAttachmentData',
    'AttachmentData',
    'ChannelMentionData',
    'AllowedMentionsData',
    'RoleSubscriptionData',
    'HasMoreListThreadsData',
)


# https://discord.com/developers/docs/resources/channel#channel-object-channel-structure


class PartialChannelData(TypedDict):
    id: 'discord_typings.Snowflake'
    name: str
    type: Literal[0, 1, 2, 3, 4, 5, 6, 10, 11, 12, 13, 14, 15, 16]
    permissions: NotRequired[str]


class TextChannelData(TypedDict):
    id: 'discord_typings.Snowflake'
    type: Literal[0]
    guild_id: NotRequired['discord_typings.Snowflake']
    position: int
    permission_overwrites: List['discord_typings.PermissionOverwriteData']
    name: str
    topic: Optional[str]
    nsfw: bool
    last_message_id: Optional['discord_typings.Snowflake']
    rate_limit_per_user: int
    parent_id: Optional['discord_typings.Snowflake']
    last_pin_timestamp: NotRequired[Optional[str]]
    default_auto_archive_duration: NotRequired[int]
    flags: int


class NewsChannelData(TypedDict):
    id: 'discord_typings.Snowflake'
    type: Literal[5]
    guild_id: NotRequired['discord_typings.Snowflake']
    position: int
    permission_overwrites: List['discord_typings.PermissionOverwriteData']
    name: str
    topic: Optional[str]
    nsfw: bool
    last_message_id: Optional['discord_typings.Snowflake']
    parent_id: Optional['discord_typings.Snowflake']
    last_pin_timestamp: NotRequired[Optional[str]]
    default_auto_archive_duration: NotRequired[int]
    flags: int


class DMChannelData(TypedDict):
    id: 'discord_typings.Snowflake'
    type: Literal[1]
    last_message_id: Optional['discord_typings.Snowflake']
    recipients: List['discord_typings.UserData']
    last_pin_timestamp: NotRequired[Optional[str]]
    flags: int


class GroupDMChannelData(TypedDict):
    id: 'discord_typings.Snowflake'
    type: Literal[3]
    name: str
    last_message_id: Optional['discord_typings.Snowflake']
    recipients: List['discord_typings.UserData']
    icon: Optional[str]
    owner_id: 'discord_typings.Snowflake'
    application_id: NotRequired['discord_typings.Snowflake']
    managed: bool
    last_pin_timestamp: NotRequired[Optional[str]]
    flags: int


class ThreadChannelData(TypedDict):
    id: 'discord_typings.Snowflake'
    type: Literal[10, 11, 12]
    guild_id: NotRequired['discord_typings.Snowflake']
    name: str
    last_message_id: Optional['discord_typings.Snowflake']
    rate_limit_per_user: int
    owner_id: 'discord_typings.Snowflake'
    parent_id: Optional['discord_typings.Snowflake']
    last_pin_timestamp: NotRequired[Optional[str]]
    message_count: int
    member_count: int
    thread_metadata: 'discord_typings.ThreadMetadataData'
    member: NotRequired['discord_typings.ThreadMemberData']
    flags: int
    total_messages_sent: int
    applied_tags: NotRequired[List['discord_typings.Snowflake']]


class VoiceChannelData(TypedDict):
    id: 'discord_typings.Snowflake'
    type: Literal[2, 13]
    guild_id: NotRequired['discord_typings.Snowflake']
    position: int
    permission_overwrites: List['discord_typings.PermissionOverwriteData']
    name: str
    nsfw: bool
    last_message_id: NotRequired[Optional['discord_typings.Snowflake']]
    bitrate: int
    user_limit: int
    parent_id: Optional['discord_typings.Snowflake']
    last_pin_timestamp: NotRequired[Optional[str]]
    rtc_region: Optional[str]
    video_quality_mode: NotRequired['discord_typings.VideoQualityModes']
    flags: int


class CategoryChannelData(TypedDict):
    id: 'discord_typings.Snowflake'
    type: Literal[4]
    guild_id: NotRequired['discord_typings.Snowflake']
    position: int
    permission_overwrites: List['discord_typings.PermissionOverwriteData']
    name: str
    nsfw: bool
    flags: int


class ForumChannelData(TypedDict):
    id: 'discord_typings.Snowflake'
    type: Literal[15]
    guild_id: NotRequired['discord_typings.Snowflake']
    position: int
    permission_overwrites: List['discord_typings.PermissionOverwriteData']
    name: str
    topic: Optional[str]
    nsfw: bool
    last_message_id: Optional['discord_typings.Snowflake']
    rate_limit_per_user: int
    default_auto_archive_duration: NotRequired[int]
    flags: int
    available_tags: List['discord_typings.ForumTagData']
    default_reaction_emoji: Optional['discord_typings.DefaultReactionData']
    default_thread_rate_limit_per_user: int
    default_sort_order: Optional['discord_typings.SortOrderTypes']
    default_forum_layout: 'discord_typings.ForumLayoutTypes'


class MediaChannelData(TypedDict):
    id: 'discord_typings.Snowflake'
    type: Literal[16]
    guild_id: NotRequired['discord_typings.Snowflake']
    position: int
    permission_overwrites: List['discord_typings.PermissionOverwriteData']
    name: str
    topic: Optional[str]
    nsfw: bool
    last_message_id: Optional['discord_typings.Snowflake']
    rate_limit_per_user: int
    default_auto_archive_duration: NotRequired[int]
    flags: int
    available_tags: List['discord_typings.ForumTagData']
    default_reaction_emoji: Optional['discord_typings.DefaultReactionData']
    default_thread_rate_limit_per_user: int
    default_sort_order: Optional['discord_typings.SortOrderTypes']
    default_forum_layout: 'discord_typings.ForumLayoutTypes'


ChannelData = Union[
    TextChannelData, NewsChannelData, DMChannelData, GroupDMChannelData,
    ThreadChannelData, VoiceChannelData, CategoryChannelData, ForumChannelData,
    MediaChannelData,
]


# https://discord.com/developers/docs/resources/channel#channel-object-channel-types


ChannelTypes = Literal[0, 1, 2, 3, 4, 5, 10, 11, 12, 13, 14, 15]


# https://discord.com/developers/docs/resources/channel#channel-object-video-quality-modes


VideoQualityModes = Literal[1, 2]


# https://discord.com/developers/docs/resources/channel#channel-object-sort-order-types


SortOrderTypes = Literal[0, 1]


# https://discord.com/developers/docs/resources/channel#channel-object-forum-layout-types


ForumLayoutTypes = Literal[0, 1, 2]


# https://discord.com/developers/docs/resources/channel#message-object-message-structure


class _ChannelMessageData(TypedDict):
    id: 'discord_typings.Snowflake'
    channel_id: 'discord_typings.Snowflake'
    author: 'discord_typings.UserData'
    content: str
    timestamp: str
    edited_timestamp: Optional[str]
    tts: bool
    mention_everyone: bool
    mentions: Union[List['discord_typings.UserData'], List['discord_typings.UserMentionData']]
    mention_roles: List['discord_typings.Snowflake']
    mention_channels: NotRequired[List['discord_typings.ChannelMentionData']]
    attachments: List['discord_typings.AttachmentData']
    embeds: List['discord_typings.EmbedData']
    reactions: NotRequired[List['discord_typings.MessageReactionData']]
    nonce: NotRequired[Union[int, str]]
    pinned: bool
    webhook_id: NotRequired['discord_typings.Snowflake']
    type: 'discord_typings.MessageTypes'
    activity: NotRequired['discord_typings.MessageActivityData']
    application: NotRequired['discord_typings.ApplicationData']
    application_id: NotRequired['discord_typings.Snowflake']
    message_reference: NotRequired['discord_typings.MessageReferenceData']
    flags: NotRequired[int]
    referenced_message: NotRequired[Optional['discord_typings.MessageData']]
    interaction_metadata: NotRequired['discord_typings.MessageInteractionMetadataData']
    interaction: NotRequired['discord_typings.MessageInteractionData']
    thread: NotRequired['discord_typings.ThreadChannelData']
    components: NotRequired[List['discord_typings.ComponentData']]
    sticker_items: NotRequired[List['discord_typings.StickerItemData']]
    position: NotRequired[int]
    role_subscription_data: NotRequired['discord_typings.RoleSubscriptionData']
    resolved: NotRequired['discord_typings.ResolvedInteractionDataData']


class _GuildMessageData(_ChannelMessageData):
    guild_id: 'discord_typings.Snowflake'
    member: 'discord_typings.GuildMemberData'


MessageData = Union[_ChannelMessageData, _GuildMessageData]


class UserMentionData(discord_typings.UserData):
    member: 'discord_typings.GuildMemberData'


# https://discord.com/developers/docs/resources/channel#message-object-message-types


MessageTypes = Literal[
    0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 14, 15, 16, 17, 18, 19, 20,
    21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32
]


# https://discord.com/developers/docs/resources/channel#message-object-message-activity-structure


class MessageActivityData(TypedDict):
    type: 'discord_typings.MessageActivityTypes'
    party_id: NotRequired[str]


# https://discord.com/developers/docs/resources/channel#message-object-message-activity-types


MessageActivityTypes = Literal[1, 2, 3, 5]


# https://discord.com/developers/docs/resources/channel#message-interaction-metadata-object


class MessageInteractionMetadataData(TypedDict):
    id: 'discord_typings.Snowflake'
    type: 'discord_typings.InteractionTypes'
    user_id: 'discord_typings.Snowflake'
    authorizing_integration_owners: Dict[
        'discord_typings.ApplicationIntegrationTypes',
        'discord_typings.Snowflake'
    ]
    original_response_message_id: NotRequired['discord_typings.Snowflake']
    interacted_message_id: NotRequired['discord_typings.Snowflake']
    triggering_interaction_metadata: NotRequired['MessageInteractionMetadataData']


# https://discord.com/developers/docs/resources/channel#message-reference-object-message-reference-structure


class MessageReferenceData(TypedDict):
    message_id: NotRequired['discord_typings.Snowflake']
    # Note: This will always be sent when receiving a message reference.
    channel_id: NotRequired['discord_typings.Snowflake']
    guild_id: NotRequired['discord_typings.Snowflake']
    fail_if_not_exists: NotRequired[bool]


# https://discord.com/developers/docs/resources/channel#followed-channel-object-followed-channel-structure


class FollowedChannelData(TypedDict):
    channel_id: 'discord_typings.Snowflake'
    webhook_id: 'discord_typings.Snowflake'


# https://discord.com/developers/docs/resources/channel#reaction-object-reaction-structure


class MessageReactionData(TypedDict):
    count: int
    count_details: 'discord_typings.ReactionCountDetailsData'
    me: bool
    me_burst: bool
    emoji: 'discord_typings.EmojiData'
    burst_colors: List[str]


# https://discord.com/developers/docs/resources/channel#reaction-count-details-object


class ReactionCountDetailsData(TypedDict):
    burst: int
    normal: int


# https://discord.com/developers/docs/resources/channel#overwrite-object-overwrite-structure


class PermissionOverwriteData(TypedDict):
    id: 'discord_typings.Snowflake'
    type: Literal[0, 1]
    allow: str
    deny: str


# https://discord.com/developers/docs/resources/channel#thread-metadata-object-thread-metadata-structure


class ThreadMetadataData(TypedDict):
    archived: bool
    auto_archive_duration: int
    archive_timestamp: str
    locked: bool
    invitable: NotRequired[bool]
    create_timestamp: NotRequired[Optional[str]]


# https://discord.com/developers/docs/resources/channel#thread-member-object-thread-member-structure


class ThreadMemberData(TypedDict):
    id: NotRequired['discord_typings.Snowflake']
    user_id: NotRequired['discord_typings.Snowflake']
    member: NotRequired['discord_typings.GuildMemberData']
    join_timestamp: str
    flags: int


# https://discord.com/developers/docs/resources/channel#default-reaction-object-default-reaction-structure


class DefaultReactionData(TypedDict):
    emoji_name: Optional[str]
    emoji_id: Optional['discord_typings.Snowflake']


# https://discord.com/developers/docs/resources/channel#forum-tag-object-forum-tag-structure


class ForumTagData(TypedDict):
    id: 'discord_typings.Snowflake'
    name: str
    moderated: bool
    emoji_id: Optional['discord_typings.Snowflake']
    emoji_name: Optional['discord_typings.Snowflake']


# https://discord.com/developers/docs/resources/channel#embed-object-embed-structure


class EmbedData(TypedDict):
    title: NotRequired[str]
    type: NotRequired[Literal['rich', 'image', 'video', 'gifv', 'article', 'link']]
    description: NotRequired[str]
    url: NotRequired[str]
    timestamp: NotRequired[str]
    color: NotRequired[int]
    footer: NotRequired['discord_typings.EmbedFooterData']
    image: NotRequired['discord_typings.EmbedImageData']
    thumbnail: NotRequired['discord_typings.EmbedThumbnailData']
    video: NotRequired['discord_typings.EmbedVideoData']
    provider: NotRequired['discord_typings.EmbedProviderData']
    author: NotRequired['discord_typings.EmbedAuthorData']
    fields: NotRequired[List['discord_typings.EmbedFieldData']]


# https://discord.com/developers/docs/resources/channel#embed-object-embed-thumbnail-structure


class EmbedThumbnailData(TypedDict):
    url: str
    proxy_url: NotRequired[str]
    height: NotRequired[int]
    width: NotRequired[int]


# https://discord.com/developers/docs/resources/channel#embed-object-embed-video-structure


class EmbedVideoData(TypedDict):
    url: NotRequired[str]
    proxy_url: NotRequired[str]
    height: NotRequired[int]
    width: NotRequired[int]


# https://discord.com/developers/docs/resources/channel#embed-object-embed-image-structure


class EmbedImageData(TypedDict):
    url: str
    proxy_url: NotRequired[str]
    height: NotRequired[int]
    width: NotRequired[int]


# https://discord.com/developers/docs/resources/channel#embed-object-embed-provider-structure


class EmbedProviderData(TypedDict):
    name: NotRequired[str]
    url: NotRequired[str]


# https://discord.com/developers/docs/resources/channel#embed-object-embed-author-structure


class EmbedAuthorData(TypedDict):
    name: str
    url: NotRequired[str]
    icon_url: NotRequired[str]
    proxy_icon_url: NotRequired[str]


# https://discord.com/developers/docs/resources/channel#embed-object-embed-footer-structure


class EmbedFooterData(TypedDict):
    text: str
    icon_url: NotRequired[str]
    proxy_icon_url: NotRequired[str]


# https://discord.com/developers/docs/resources/channel#embed-object-embed-field-structure


class EmbedFieldData(TypedDict):
    name: str
    value: str
    inline: NotRequired[bool]


# https://discord.com/developers/docs/resources/channel#attachment-object-attachment-structure


class PartialAttachmentData(TypedDict):
    id: 'discord_typings.Snowflake'
    filename: NotRequired[str]
    description: NotRequired[str]


class AttachmentData(TypedDict):
    id: 'discord_typings.Snowflake'
    filename: str
    description: NotRequired[str]
    content_type: NotRequired[str]
    size: int
    url: str
    proxy_url: str
    height: NotRequired[Optional[int]]
    width: NotRequired[Optional[int]]
    ephemeral: NotRequired[bool]
    duration_secs: NotRequired[float]
    waveform: NotRequired[str]
    flags: NotRequired[int]


# https://discord.com/developers/docs/resources/channel#channel-mention-object-channel-mention-structure


class ChannelMentionData(TypedDict):
    id: 'discord_typings.Snowflake'
    guild_id: 'discord_typings.Snowflake'
    type: Literal[0, 1, 2, 3, 4, 5, 6, 10, 11, 12, 13]
    name: str


# https://discord.com/developers/docs/resources/channel#allowed-mentions-object-allowed-mentions-structure


class AllowedMentionsData(TypedDict):
    parse: NotRequired[List[Literal['roles', 'users', 'everyone']]]
    roles: NotRequired[List['discord_typings.Snowflake']]
    users: NotRequired[List['discord_typings.Snowflake']]
    replied_user: NotRequired[bool]


# https://discord.com/developers/docs/resources/channel#role-subscription-data-object-role-subscription-data-object-structure


class RoleSubscriptionData(TypedDict):
    role_subscription_listing_id: 'discord_typings.Snowflake'
    tier_name: str
    total_months_subscribed: int
    is_renewal: bool


# https://discord.com/developers/docs/resources/channel#list-public-archived-threads-response-body


class HasMoreListThreadsData(TypedDict):
    threads: List[ThreadChannelData]
    members: List[ThreadMemberData]
    has_more: bool



from typing import Optional

from typing_extensions import Literal, NotRequired, TypedDict

import discord_typings

__all__ = (
    'WebhookData',
    'WebhookTypes',
)


# https://discord.com/developers/docs/resources/webhook#webhook-object-webhook-structure


class WebhookData(TypedDict):
    id: 'discord_typings.Snowflake'
    type: 'discord_typings.WebhookTypes'
    guild_id: NotRequired[Optional['discord_typings.Snowflake']]
    channel_id: Optional['discord_typings.Snowflake']
    user: NotRequired['discord_typings.UserData']
    name: Optional[str]
    avatar: Optional[str]
    token: NotRequired[str]
    application_id: Optional['discord_typings.Snowflake']
    source_guild: NotRequired['discord_typings.GuildData']
    source_channel: NotRequired['discord_typings.PartialChannelData']
    url: NotRequired[str]


# https://discord.com/developers/docs/resources/webhook#webhook-object-webhook-types


WebhookTypes = Literal[1, 2, 3]



from typing import Optional

from typing_extensions import TypedDict

import discord_typings

__all__ = ('GuildTemplateData',)


# https://discord.com/developers/docs/resources/guild-template#guild-template-object


class GuildTemplateData(TypedDict):
    code: str
    name: str
    description: Optional[str]
    usage_count: int
    creator_id: 'discord_typings.Snowflake'
    creator: 'discord_typings.UserData'
    created_at: str
    updated_at: str
    source_guild_id: 'discord_typings.Snowflake'
    serialized_source_guild: 'discord_typings.GuildData'
    is_dirty: Optional[bool]



from typing import Dict, List, Union

from typing_extensions import Literal, TypedDict

__all__ = (
    'Snowflake',
    'HTTPErrorData',
    'InnerHTTPErrorsData',
    'NestedHTTPErrorsData',
    'HTTPErrorResponseData',
    'Locales',
)


# Somewhat useless but good to distinguish the difference, perhaps in the
# future Python's typing system will have something similar to TypeScript's
# "Template Literal Types" which is able to describe a snowflake.
# Discord handles integers on endpoints but all snowflakes received by the API
# will be strings.
# https://discord.com/developers/docs/reference#snowflakes
Snowflake = Union[str, int]


# https://discord.com/developers/docs/reference#error-messages


class HTTPErrorData(TypedDict):
    code: str
    message: str


class InnerHTTPErrorsData(TypedDict):
    _errors: List[HTTPErrorData]


NestedHTTPErrorsData = Union[
    Dict[str, 'NestedHTTPErrorsData'],
    InnerHTTPErrorsData
]


class HTTPErrorResponseData(TypedDict):
    code: int
    errors: NestedHTTPErrorsData
    message: str


# https://discord.com/developers/docs/reference#locales


Locales = Literal[
    'id',
    'da',
    'de',
    'en-GB',
    'en-US',
    'en-ES',
    'es-419',
    'fr',
    'hr',
    'it',
    'lt',
    'hu',
    'nl',
    'no',
    'pl',
    'pt-BR',
    'ro',
    'fi',
    'sv-SE',
    'vi',
    'tr',
    'cs',
    'el',
    'bg',
    'ru',
    'uk',
    'hi',
    'th',
    'zh-CN',
    'ja',
    'zh-TW',
    'ko',
]



from typing import List, Optional, Union

from typing_extensions import Literal, NotRequired, TypedDict

import discord_typings

__all__ = (
    'PartialGuildData',
    'GuildData',
    'DefaultMessageNotificationLevels',
    'ExplicitContentFilterLevels',
    'MFALevels',
    'VerificationLevels',
    'GuildNSFWLevels',
    'PremiumTiers',
    'GuildFeatures',
    'UnavailableGuildData',
    'GuildPreviewData',
    'GuildWidgetSettingsData',
    'GuildWidgetData',
    'GuildMemberData',
    'StreamingIntegrationData',
    'DiscordIntegrationData',
    'IntegrationData',
    'IntegrationExpireBehaviors',
    'IntegrationAccountData',
    'IntegrationApplicationData',
    'BanData',
    'WelcomeScreenData',
    'WelcomeChannelData',
    'GuildOnboardingData',
    'GuildOnboardingPromptsData',
    'GuildOnboardingPromptOptionData',
    'GuildOnboardingModes',
    'GuildOnboardingPromptTypes',
    'ChannelPositionData',
    'ListThreadsData',
    'RolePositionData',
    'RoleData',
    'RoleTagsData',
    'PartialGuildData',
)


# https://discord.com/developers/docs/resources/user#get-current-user-guilds


class PartialGuildData(TypedDict):
    id: 'discord_typings.Snowflake'
    name: str
    icon: Optional[str]
    owner: bool
    permissions: str
    features: List['discord_typings.GuildFeatures']
    approximate_member_count: NotRequired[int]
    approximate_presence_count: NotRequired[int]


# https://discord.com/developers/docs/resources/guild#guild-object


class GuildData(TypedDict):
    id: 'discord_typings.Snowflake'
    name: str
    icon: Optional[str]
    icon_hash: NotRequired[Optional[str]]
    splash: Optional[str]
    discovery_splash: Optional[str]
    owner: NotRequired[bool]
    owner_id: 'discord_typings.Snowflake'
    permissions: NotRequired[str]
    afk_channel_id: Optional['discord_typings.Snowflake']
    afk_timeout: int
    widget_enabled: NotRequired[bool]
    widget_channel_id: NotRequired[Optional['discord_typings.Snowflake']]
    verification_level: 'discord_typings.VerificationLevels'
    default_message_notifications: 'discord_typings.DefaultMessageNotificationLevels'
    explicit_content_filter: 'discord_typings.ExplicitContentFilterLevels'
    roles: List['discord_typings.RoleData']
    emojis: List['discord_typings.EmojiData']
    features: List['discord_typings.GuildFeatures']
    mfa_level: 'discord_typings.MFALevels'
    application_id: Optional[str]
    system_channel_id: Optional['discord_typings.Snowflake']
    system_channel_flags: int
    rules_channel_id: Optional['discord_typings.Snowflake']
    max_presences: NotRequired[Optional[int]]
    max_members: NotRequired[int]
    vanity_url_code: Optional[str]
    description: Optional[str]
    banner: Optional[str]
    premium_tier: 'discord_typings.PremiumTiers'
    premium_subscription_count: NotRequired[int]
    preferred_locale: 'discord_typings.Locales'
    public_updates_channel_id: Optional['discord_typings.Snowflake']
    max_video_channel_users: NotRequired[int]
    max_stage_video_channel_users: NotRequired[int]
    approximate_member_count: NotRequired[int]
    approximate_presence_count: NotRequired[int]
    welcome_screen: NotRequired['discord_typings.WelcomeScreenData']
    nsfw_level: 'discord_typings.GuildNSFWLevels'
    stickers: NotRequired[List['discord_typings.StickerData']]
    premium_progress_bar_enabled: bool
    safety_alerts_channel_id: Optional['discord_typings.Snowflake']


# https://discord.com/developers/docs/resources/guild#guild-object-default-message-notification-level


DefaultMessageNotificationLevels = Literal[0, 1]


# https://discord.com/developers/docs/resources/guild#guild-object-explicit-content-filter-level


ExplicitContentFilterLevels = Literal[0, 1, 2]


# https://discord.com/developers/docs/resources/guild#guild-object-mfa-level


MFALevels = Literal[0, 1]


# https://discord.com/developers/docs/resources/guild#guild-object-verification-level


VerificationLevels = Literal[0, 1, 2, 3, 4]


# https://discord.com/developers/docs/resources/guild#guild-object-guild-nsfw-level


GuildNSFWLevels = Literal[0, 1, 2, 3]


# https://discord.com/developers/docs/resources/guild#guild-object-premium-tier


PremiumTiers = Literal[0, 1, 2, 3]


# https://discord.com/developers/docs/resources/guild#guild-object-guild-features


GuildFeatures = Literal[
    'ANIMATED_BANNER',
    'ANIMATED_ICON',
    'APPLICATION_COMMAND_PERMISSIONS_V2',
    'AUTO_MODERATION',
    'BANNER',
    'COMMUNITY',
    'CREATOR_MONETIZABLE_PROVISIONAL',
    'CREATOR_STORE_PAGE',
    'DEVELOPER_SUPPORT_SERVER',
    'DISCOVERABLE',
    'FEATUREABLE',
    'INVITES_DISABLED',
    'INVITE_SPLASH',
    'MEMBER_VERIFICATION_GATE_ENABLED',
    'MORE_STICKERS',
    'NEWS',
    'PARTNERED',
    'PREVIEW_ENABLED',
    'RAID_ALERTS_DISABLED',
    'ROLE_ICONS',
    'ROLE_SUBSCRIPTIONS_AVAILABLE_FOR_PURCHASE',
    'ROLE_SUBSCRIPTIONS_ENABLED',
    'TICKETED_EVENTS_ENABLED',
    'VANITY_URL',
    'VERIFIED',
    'VIP_REGIONS',
    'WELCOME_SCREEN_ENABLED',
]


# https://discord.com/developers/docs/resources/guild#unavailable-guild-object-example-unavailable-guild


class UnavailableGuildData(TypedDict):
    id: str
    unavailable: bool


# https://discord.com/developers/docs/resources/guild#guild-preview-object-guild-preview-structure


class GuildPreviewData(TypedDict):
    id: 'discord_typings.Snowflake'
    name: str
    icon: Optional[str]
    splash: Optional[str]
    discovery_splash: Optional[str]
    emojis: List['discord_typings.EmojiData']
    features: List['discord_typings.GuildFeatures']
    approximate_member_count: int
    approximate_presence_count: int
    description: Optional[str]
    stickers: List['discord_typings.StickerData']


# https://discord.com/developers/docs/resources/guild#guild-widget-object-guild-widget-structure


class GuildWidgetSettingsData(TypedDict):
    enabled: bool
    channel_id: Optional['discord_typings.Snowflake']


# https://discord.com/developers/docs/resources/guild#guild-widget-object


class GuildWidgetData(TypedDict):
    id: 'discord_typings.Snowflake'
    name: str
    instant_invite: Optional[str]
    channels: List['discord_typings.PartialChannelData']
    members: List['discord_typings.UserData']
    presence_count: int


# https://discord.com/developers/docs/resources/guild#guild-member-object-guild-member-structure


class GuildMemberData(TypedDict):
    user: NotRequired['discord_typings.UserData']
    nick: NotRequired[Optional[str]]
    avatar: NotRequired[Optional[str]]
    roles: List['discord_typings.Snowflake']
    joined_at: str
    premium_since: NotRequired[Optional[str]]
    deaf: bool
    mute: bool
    flags: int
    pending: NotRequired[bool]
    permissions: NotRequired[str]
    communication_disabled_until: NotRequired[Optional[str]]


# https://discord.com/developers/docs/resources/guild#integration-object-integration-structure


class StreamingIntegrationData(TypedDict):
    id: 'discord_typings.Snowflake'
    name: str
    type: Literal['twitch', 'youtube', 'discord']
    enabled: bool
    syncing: bool
    role_id: NotRequired['discord_typings.Snowflake']
    enable_emoticons: bool
    expire_behavior: 'discord_typings.IntegrationExpireBehaviors'
    expire_grace_period: int
    user: NotRequired['discord_typings.UserData']
    account: 'discord_typings.IntegrationAccountData'
    synced_at: str
    subscriber_count: int
    revoked: bool
    application: 'discord_typings.IntegrationApplicationData'
    scopes: NotRequired[List['discord_typings.OAuth2Scopes']]


class DiscordIntegrationData(TypedDict):
    id: 'discord_typings.Snowflake'
    name: str
    type: Literal['discord']
    enabled: bool
    user: NotRequired['discord_typings.UserData']
    account: 'discord_typings.IntegrationAccountData'
    application: NotRequired['discord_typings.IntegrationApplicationData']
    scopes: NotRequired[List['discord_typings.OAuth2Scopes']]


IntegrationData = Union[StreamingIntegrationData, DiscordIntegrationData]


# https://discord.com/developers/docs/resources/guild#integration-object-integration-expire-behaviors


IntegrationExpireBehaviors = Literal[0, 1]


# https://discord.com/developers/docs/resources/guild#integration-account-object-integration-account-structure


class IntegrationAccountData(TypedDict):
    id: str
    name: str


# https://discord.com/developers/docs/resources/guild#integration-application-object-integration-application-structure


class IntegrationApplicationData(TypedDict):
    id: 'discord_typings.Snowflake'
    name: str
    icon: Optional[str]
    description: str
    summary: str
    bot: NotRequired[Optional['discord_typings.UserData']]


# https://discord.com/developers/docs/resources/guild#ban-object-ban-structure


class BanData(TypedDict):
    reason: Optional[str]
    user: 'discord_typings.UserData'


# https://discord.com/developers/docs/resources/guild#welcome-screen-object-welcome-screen-structure


class WelcomeScreenData(TypedDict):
    description: Optional[str]
    welcome_channels: List['discord_typings.WelcomeChannelData']


# https://discord.com/developers/docs/resources/guild#welcome-screen-object-welcome-screen-channel-structure


class WelcomeChannelData(TypedDict):
    channel_id: 'discord_typings.Snowflake'
    description: str
    emoji_id: Optional['discord_typings.Snowflake']
    emoji_name: Optional[str]


# https://discord.com/developers/docs/resources/guild#guild-onboarding-object


class GuildOnboardingData(TypedDict):
    guild_id: 'discord_typings.Snowflake'
    prompts: List['discord_typings.GuildOnboardingPromptsData']
    default_channel_ids: List['discord_typings.Snowflake']
    enabled: bool
    mode: 'discord_typings.GuildOnboardingModes'


# https://discord.com/developers/docs/resources/guild#guild-onboarding-object-onboarding-prompt-structure


class GuildOnboardingPromptsData(TypedDict):
    id: 'discord_typings.Snowflake'
    type: 'discord_typings.GuildOnboardingPromptTypes'
    options: List['discord_typings.GuildOnboardingPromptOptionData']
    title: str
    single_select: bool
    required: bool
    in_onboarding: bool


# https://discord.com/developers/docs/resources/guild#guild-onboarding-object-prompt-option-structure


class GuildOnboardingPromptOptionData(TypedDict):
    id: 'discord_typings.Snowflake'
    channel_ids: List['discord_typings.Snowflake']
    role_ids: List['discord_typings.Snowflake']
    emoji: NotRequired['discord_typings.EmojiData']
    emoji_id: NotRequired['discord_typings.Snowflake']
    emoji_name: NotRequired[str]
    emoji_animated: NotRequired[bool]
    title: str
    description: Optional[str]


# https://discord.com/developers/docs/resources/guild#guild-onboarding-object-onboarding-mode


GuildOnboardingModes = Literal[0, 1]


# https://discord.com/developers/docs/resources/guild#guild-onboarding-object-prompt-types


GuildOnboardingPromptTypes = Literal[0, 1]


# https://discord.com/developers/docs/resources/guild#modify-guild-channel-positions-json-params


class ChannelPositionData(TypedDict):
    id: 'discord_typings.Snowflake'
    position: Optional[int]
    lock_permissions: Optional[bool]
    parent_id: Optional['discord_typings.Snowflake']


# https://discord.com/developers/docs/resources/guild#list-active-threads-response-body


class ListThreadsData(TypedDict):
    threads: List['discord_typings.ThreadChannelData']
    members: List['discord_typings.ThreadMemberData']


# https://discord.com/developers/docs/resources/guild#modify-guild-role-positions-json-params


class RolePositionData(TypedDict):
    id: 'discord_typings.Snowflake'
    position: NotRequired[Optional[int]]


# https://discord.com/developers/docs/topics/permissions#role-object-role-structure


class RoleData(TypedDict):
    id: 'discord_typings.Snowflake'
    name: str
    color: int
    hoist: bool
    icon: NotRequired[Optional[str]]
    unicode_emoji: NotRequired[Optional[str]]
    position: int
    permissions: str
    managed: bool
    mentionable: bool
    tags: NotRequired['discord_typings.RoleTagsData']
    flags: int


# https://discord.com/developers/docs/topics/permissions#role-object-role-tags-structure


class RoleTagsData(TypedDict):
    bot_id: NotRequired['discord_typings.Snowflake']
    integration_id: NotRequired['discord_typings.Snowflake']
    premium_subscriber: NotRequired[None]
    subscription_listing_id: NotRequired['discord_typings.Snowflake']
    available_for_purchase: NotRequired[None]
    guild_connections: NotRequired[None]



from typing import Dict, List, Optional, Union

from typing_extensions import Literal, NotRequired, TypedDict

__all__ = (
    'UserData',
    'UserPremiumTypes',
    'ConnectionData',
    'ConnectionTypes',
    'ConnectionVisibilityTypes',
    'ApplicationRoleConnectionData',
)

import discord_typings

# https://discord.com/developers/docs/resources/user#user-object-user-structure


class UserData(TypedDict):
    id: 'discord_typings.Snowflake'
    username: str
    discriminator: Union[str, Literal["0"]]
    global_name: Optional[str]
    avatar: Optional[str]
    bot: NotRequired[bool]
    system: NotRequired[bool]
    mfa_enabled: NotRequired[bool]
    banner: NotRequired[Optional[str]]
    accent_color: NotRequired[Optional[int]]
    locale: NotRequired['discord_typings.Locales']
    verified: NotRequired[bool]
    email: NotRequired[Optional[str]]
    flags: NotRequired[int]
    premium_type: NotRequired['discord_typings.UserPremiumTypes']
    public_flags: NotRequired[int]
    avatar_decoration: NotRequired[Optional[str]]


# https://discord.com/developers/docs/resources/user#user-object-premium-types


UserPremiumTypes = Literal[0, 1, 2, 3]


# https://discord.com/developers/docs/resources/user#connection-object


class ConnectionData(TypedDict):
    id: str
    name: str
    type: 'discord_typings.ConnectionTypes'
    revoked: NotRequired[bool]
    integrations: NotRequired[List['discord_typings.IntegrationData']]
    verified: bool
    friend_sync: bool
    show_activity: bool
    two_way_link: bool
    visibility: 'discord_typings.ConnectionVisibilityTypes'


# https://discord.com/developers/docs/resources/user#connection-object-services


ConnectionTypes = Literal[
    'battlenet',
    'ebay',
    'epicgames',
    'facebook',
    'github',
    'instagram',
    'leagueoflegends',
    'paypal',
    'playstation',
    'reddit',
    'riotgames',
    'spotify',
    'skype',
    'steam',
    'tiktok',
    'twitch',
    'twitter',
    'xbox',
    'youtube',
]


# https://discord.com/developers/docs/resources/user#connection-object-visibility-types


ConnectionVisibilityTypes = Literal[0, 1]


# https://discord.com/developers/docs/resources/user#application-role-connection-object


class ApplicationRoleConnectionData(TypedDict):
    platform_name: Optional[str]
    platform_username: Optional[str]
    metadata: Dict[str, str]



from typing import Optional

from typing_extensions import NotRequired, TypedDict

import discord_typings

__all__ = (
    'VoiceStateData',
    'VoiceRegionData',
)


# https://discord.com/developers/docs/resources/voice#voice-state-object-voice-state-structure


class VoiceStateData(TypedDict):
    guild_id: NotRequired['discord_typings.Snowflake']
    channel_id: Optional['discord_typings.Snowflake']
    user_id: 'discord_typings.Snowflake'
    member: NotRequired['discord_typings.GuildMemberData']
    session_id: str
    deaf: bool
    mute: bool
    self_deaf: bool
    self_mute: bool
    self_stream: NotRequired[bool]
    self_video: bool
    suppress: bool
    request_to_speak_timestamp: Optional[str]


# https://discord.com/developers/docs/resources/voice#voice-region-object-voice-region-structure


class VoiceRegionData(TypedDict):
    id: str
    name: str
    optimal: bool
    deprecated: bool
    custom: bool



from typing import List, Union

from typing_extensions import Literal, NotRequired, TypedDict

import discord_typings

__all__ = (
    'ComponentTypes',
    'ActionRowData',
    'ButtonComponentData',
    'ButtonStyles',
    'StringSelectMenuComponentData',
    'UserSelectMenuComponentData',
    'RoleSelectMenuComponentData',
    'MentionableSelectMenuComponentData',
    'ChannelSelectMenuComponentData',
    'SelectMenuComponentData',
    'SelectMenuOptionData',
    'SelectMenuDefaultValueData',
    'SelectMenuDefaultValueTypes',
    'TextInputComponentData',
    'TextInputStyles',
    'ComponentData',
)


# https://discord.com/developers/docs/interactions/message-components#component-object-component-types


ComponentTypes = Literal[1, 2, 3, 4, 5, 6, 7, 8]


# https://discord.com/developers/docs/interactions/message-components#action-rows


class ActionRowData(TypedDict):
    type: Literal[1]
    components: List[Union[
        'discord_typings.ButtonComponentData',
        'discord_typings.SelectMenuComponentData',
        'discord_typings.TextInputComponentData',
    ]]


# https://discord.com/developers/docs/interactions/message-components#button-object-button-structure


# The whole button object itself is split into NonLinkButtonComponent and
# LinkButtonComponent because link buttons need to have the link-style.
class _NonLinkButtonComponentData(TypedDict):
    type: Literal[2]
    style: Literal[1, 2, 3, 4]
    label: NotRequired[str]
    emoji: NotRequired['discord_typings.EmojiData']
    custom_id: str
    disabled: NotRequired[bool]


class _LinkButtonComponentData(TypedDict):
    type: Literal[2]
    style: Literal[5]
    label: NotRequired[str]
    emoji: NotRequired['discord_typings.EmojiData']
    url: str
    disabled: NotRequired[bool]


ButtonComponentData = Union[_NonLinkButtonComponentData, _LinkButtonComponentData]

# https://discord.com/developers/docs/interactions/message-components#button-object-button-styles


ButtonStyles = Literal[1, 2, 3, 4, 5]


# https://discord.com/developers/docs/interactions/message-components#select-menu-object-select-menu-structure


class StringSelectMenuComponentData(TypedDict):
    type: Literal[3]
    custom_id: str
    options: List['discord_typings.SelectMenuOptionData']
    placeholder: NotRequired[str]
    min_values: NotRequired[int]
    max_values: NotRequired[int]
    disabled: NotRequired[bool]


class UserSelectMenuComponentData(TypedDict):
    type: Literal[5]
    custom_id: str
    placeholder: NotRequired[str]
    default_values: NotRequired[List['discord_typings.SelectMenuDefaultValueData']]
    min_values: NotRequired[int]
    max_values: NotRequired[int]
    disabled: NotRequired[bool]


class RoleSelectMenuComponentData(TypedDict):
    type: Literal[6]
    custom_id: str
    placeholder: NotRequired[str]
    default_values: NotRequired[List['discord_typings.SelectMenuDefaultValueData']]
    min_values: NotRequired[int]
    max_values: NotRequired[int]
    disabled: NotRequired[bool]


class MentionableSelectMenuComponentData(TypedDict):
    type: Literal[7]
    custom_id: str
    placeholder: NotRequired[str]
    default_values: NotRequired[List['discord_typings.SelectMenuDefaultValueData']]
    min_values: NotRequired[int]
    max_values: NotRequired[int]
    disabled: NotRequired[bool]


class ChannelSelectMenuComponentData(TypedDict):
    type: Literal[8]
    custom_id: str
    channel_types: List['discord_typings.ChannelTypes']
    placeholder: NotRequired[str]
    default_values: NotRequired[List['discord_typings.SelectMenuDefaultValueData']]
    min_values: NotRequired[int]
    max_values: NotRequired[int]
    disabled: NotRequired[bool]


SelectMenuComponentData = Union[
    StringSelectMenuComponentData, UserSelectMenuComponentData,
    RoleSelectMenuComponentData, MentionableSelectMenuComponentData,
    ChannelSelectMenuComponentData,
]


# https://discord.com/developers/docs/interactions/message-components#select-menu-object-select-option-structure


class SelectMenuOptionData(TypedDict):
    label: str
    value: str
    description: NotRequired[str]
    emoji: NotRequired['discord_typings.EmojiData']
    default: NotRequired[bool]


# https://discord.com/developers/docs/interactions/message-components#select-menu-object-select-default-value-structure


class SelectMenuDefaultValueData(TypedDict):
    id: 'discord_typings.Snowflake'
    type: 'discord_typings.SelectMenuDefaultValueTypes'


SelectMenuDefaultValueTypes = Literal['user', 'roles', 'channel']


# https://discord.com/developers/docs/interactions/message-components#text-inputs-text-input-structure


class TextInputComponentData(TypedDict):
    type: Literal[4]
    custom_id: str
    style: 'discord_typings.TextInputStyles'
    label: str
    min_length: NotRequired[int]
    max_length: NotRequired[int]
    required: NotRequired[bool]
    value: NotRequired[str]
    placeholder: NotRequired[str]


# https://discord.com/developers/docs/interactions/message-components#text-inputs-text-input-styles


TextInputStyles = Literal[1, 2]


ComponentData = Union[
    ActionRowData, ButtonComponentData, SelectMenuComponentData, TextInputComponentData
]



from typing import Optional

from typing_extensions import Literal, TypedDict

import discord_typings

__all__ = (
    'StageInstanceData',
    'StageInstancePrivacyLevels',
)


# https://discord.com/developers/docs/resources/stage-instance#stage-instance-object-stage-instance-structure


class StageInstanceData(TypedDict):
    id: 'discord_typings.Snowflake'
    guild_id: 'discord_typings.Snowflake'
    channel_id: 'discord_typings.Snowflake'
    topic: str
    privacy_level: 'discord_typings.StageInstancePrivacyLevels'
    discoverable_disabled: bool
    guild_scheduled_event_id: Optional['discord_typings.Snowflake']


# https://discord.com/developers/docs/resources/stage-instance#stage-instance-object-privacy-level


StageInstancePrivacyLevels = Literal[1, 2]



from typing import List, Optional

from typing_extensions import Literal, NotRequired, TypedDict

import discord_typings

__all__ = (
    'InviteData',
    'InviteTargetTypes',
    'InviteMetadata',
    'InviteStageInstanceData',
)


# https://discord.com/developers/docs/resources/invite#invite-object-invite-structure


class InviteData(TypedDict):
    code: str
    guild: NotRequired['discord_typings.PartialGuildData']
    channel: Optional['discord_typings.PartialChannelData']
    inviter: NotRequired['discord_typings.UserData']
    target_type: NotRequired['discord_typings.InviteTargetTypes']
    target_user: NotRequired['discord_typings.UserData']
    target_application: NotRequired['discord_typings.ApplicationData']
    approximate_presence_count: NotRequired[int]
    approximate_member_count: NotRequired[int]
    expires_at: NotRequired[Optional[str]]
    stage_instance: NotRequired['discord_typings.InviteStageInstanceData']
    guild_scheduled_event: NotRequired['discord_typings.GuildScheduledEventData']


# https://discord.com/developers/docs/resources/invite#invite-object-invite-target-types


InviteTargetTypes = Literal[1, 2]


# https://discord.com/developers/docs/resources/invite#invite-metadata-object-invite-metadata-structure


class InviteMetadata(InviteData):
    uses: int
    max_uses: int
    max_age: int
    temporary: bool
    created_at: str


# https://discord.com/developers/docs/resources/invite#invite-stage-instance-object-invite-stage-instance-structure


class InviteStageInstanceData(TypedDict):
    members: List['discord_typings.GuildMemberData']
    participant_count: int
    speaker_count: int
    topic: str



from typing import (
    Any, Dict, Generic, List, Mapping, Optional, Tuple, TypeVar, Union
)

from typing_extensions import Literal, NotRequired, TypeAlias, TypedDict

import discord_typings

__all__ = (
    'GetGatewayData',
    'GetGatewayBotData',
    'SessionStartLimitData',
    'IdentifyData',
    'IdentifyConnectionPropertiesData',
    'IdentifyCommand',
    'ResumeData',
    'ResumeCommand',
    'HeartbeatCommand',
    'RequestGuildMembersData',
    'RequestGuildMembersCommand',
    'UpdateVoiceStateData',
    'UpdateVoiceStateCommand',
    'UpdatePresenceData',
    'UpdatePresenceCommand',
    'HelloData',
    'HelloEvent',
    'HeartbeatACKEvent',
    'PartialApplicationData',
    'ReadyData',
    'ReadyEvent',
    'ResumedData',
    'ResumedEvent',
    'ReconnectEvent',
    'InvalidSessionEvent',
    'ApplicationCommandPermissionsUpdateData',
    'ApplicationCommandPermissionsUpdateEvent',
    'AutoModerationRuleCreateData',
    'AutoModerationRuleCreateEvent',
    'AutoModerationRuleUpdateData',
    'AutoModerationRuleUpdateEvent',
    'AutoModerationRuleDeleteData',
    'AutoModerationRuleDeleteEvent',
    'AutoModerationActionExecutionData',
    'AutoModerationActionExecutionEvent',
    'ChannelCreateData',
    'ChannelCreateEvent',
    'ChannelUpdateData',
    'ChannelUpdateEvent',
    'ChannelDeleteData',
    'ChannelDeleteEvent',
    'ThreadCreateData',
    'ThreadCreateEvent',
    'ThreadUpdateData',
    'ThreadUpdateEvent',
    'ThreadDeleteData',
    'ThreadDeleteEvent',
    'ThreadListSyncData',
    'ThreadListSyncEvent',
    'ThreadMemberUpdateData',
    'ThreadMemberUpdateEvent',
    'ThreadMembersUpdateData',
    'ThreadMembersUpdateEvent',
    'EntitlementCreateData',
    'EntitlementCreateEvent',
    'EntitlementUpdateData',
    'EntitlementUpdateEvent',
    'EntitlementDeleteData',
    'EntitlementDeleteEvent',
    'ChannelPinsUpdateData',
    'ChannelPinsUpdateEvent',
    'GuildCreateData',
    'GuildCreateEvent',
    'GuildUpdateData',
    'GuildUpdateEvent',
    'GuildDeleteData',
    'GuildDeleteEvent',
    'GuildAuditLogEntryCreateData',
    'GuildAuditLogEntryCreateEvent',
    'GuildBanAddData',
    'GuildBanAddEvent',
    'GuildBanRemoveData',
    'GuildBanRemoveEvent',
    'GuildEmojisUpdateData',
    'GuildEmojisUpdateEvent',
    'GuildStickersUpdateData',
    'GuildStickersUpdateEvent',
    'GuildIntergrationsUpdateData',
    'GuildIntergrationsUpdateEvent',
    'GuildMemberAddData',
    'GuildMemberAddEvent',
    'GuildMemberRemoveData',
    'GuildMemberRemoveEvent',
    'GuildMemberUpdateData',
    'GuildMemberUpdateEvent',
    'GuildMembersChunkData',
    'GuildMembersChunkEvent',
    'GuildRoleCreateData',
    'GuildRoleCreateEvent',
    'GuildRoleUpdateData',
    'GuildRoleUpdateEvent',
    'GuildRoleDeleteData',
    'GuildRoleDeleteEvent',
    'GuildScheduledEventCreateData',
    'GuildScheduledEventCreateEvent',
    'GuildScheduledEventUpdateData',
    'GuildScheduledEventUpdateEvent',
    'GuildScheduledEventDeleteData',
    'GuildScheduledEventDeleteEvent',
    'GuildScheduledEventUserAddData',
    'GuildScheduledEventUserAddEvent',
    'GuildScheduledEventUserRemoveData',
    'GuildScheduledEventUserRemoveEvent',
    'IntegrationCreateData',
    'IntegrationCreateEvent',
    'IntegrationUpdateData',
    'IntegrationUpdateEvent',
    'IntegrationDeleteData',
    'IntegrationDeleteEvent',
    'InviteCreateData',
    'InviteCreateEvent',
    'InviteDeleteData',
    'InviteDeleteEvent',
    'MessageCreateData',
    'MessageCreateEvent',
    'MessageUpdateData',
    'MessageUpdateEvent',
    'MessageDeleteData',
    'MessageDeleteEvent',
    'MessageDeleteBulkData',
    'MessageDeleteBulkEvent',
    'MessageReactionAddData',
    'MessageReactionAddEvent',
    'MessageReactionRemoveData',
    'MessageReactionRemoveEvent',
    'MessageReactionRemoveAllData',
    'MessageReactionRemoveAllEvent',
    'MessageReactionRemoveEmojiData',
    'MessageReactionRemoveEmojiEvent',
    'PresenceUpdateData',
    'ClientStatusData',
    'PartialActivityData',
    'ActivityData',
    'ActivityTypes',
    'ActivityTimestampsData',
    'ActivityEmojiData',
    'ActivityPartyData',
    'ActivityAssetsData',
    'ActivitySecretsData',
    'ActivityButtonData',
    'TypingStartData',
    'TypingStartEvent',
    'UserUpdateData',
    'UserUpdateEvent',
    'VoiceStateUpdateData',
    'VoiceStateUpdateEvent',
    'VoiceServerUpdateData',
    'VoiceServerUpdateEvent',
    'WebhooksUpdateData',
    'WebhooksUpdateEvent',
    'InteractionCreateData',
    'InteractionCreateEvent',
    'StageInstanceCreateData',
    'StageInstanceCreateEvent',
    'StageInstanceUpdateData',
    'StageInstanceUpdateEvent',
    'StageInstanceDeleteData',
    'StageInstanceDeleteEvent',
    'DispatchEvent',
    'GatewayCommand',
    'GatewayEvent',
)


_D = TypeVar('_D', bound='Mapping[str, Any]')
_T = TypeVar('_T', bound='str')  # Literal


# https://discord.com/developers/docs/topics/gateway-events#get-gateway-example-response


class GetGatewayData(TypedDict):
    url: str


# https://discord.com/developers/docs/topics/gateway#get-gateway-bot


class GetGatewayBotData(TypedDict):
    url: str
    shards: int
    session_start_limit: 'discord_typings.SessionStartLimitData'


# https://discord.com/developers/docs/topics/gateway#session-start-limit-object


class SessionStartLimitData(TypedDict):
    total: int
    remaining: int
    reset_after: int
    max_concurrency: int


# https://discord.com/developers/docs/topics/gateway-events#payload-structure


class GenericDispatchEvent(TypedDict, Generic[_T, _D]):
    """Generic Discord dispatch event.

    This is a generic TypedDict for creating typings for dispatch events. The
    resulting TypedDict contains the `op`, `d`, `s` and `t` fields. To get the
    typing for the inner `d` key, grab the second argument to the generic.

    The reason this doesn't have a leading underscore is to look nicer when
    previewed in editors and IDEs but the actual `GenericDispatchEvent()`
    should be considered private - use the public, typed, versions.
    """
    op: Literal[0]
    d: _D
    s: int
    t: _T


# https://discord.com/developers/docs/topics/gateway-events#identify


class IdentifyData(TypedDict):
    token: str
    properties: 'discord_typings.IdentifyConnectionPropertiesData'
    compress: NotRequired[bool]
    large_threshold: NotRequired[int]
    shard: NotRequired[Union[Tuple[int, int], List[int]]]
    presence: NotRequired['discord_typings.UpdatePresenceData']
    intents: int


class IdentifyConnectionPropertiesData(TypedDict):
    os: str
    browser: str
    device: str


class IdentifyCommand(TypedDict):
    op: Literal[2]
    d: IdentifyData


# https://discord.com/developers/docs/topics/gateway-events#resume


class ResumeData(TypedDict):
    token: str
    session_id: str
    seq: int


class ResumeCommand(TypedDict):
    op: Literal[6]
    d: ResumeData


# https://discord.com/developers/docs/topics/gateway-events#heartbeat


class HeartbeatCommand(TypedDict):
    op: Literal[1]
    d: Optional[int]


# https://discord.com/developers/docs/topics/gateway-events#request-guild-members


class _QueryRequestMembersCommand(TypedDict):
    guild_id: 'discord_typings.Snowflake'
    query: str
    limit: int
    presences: NotRequired[bool]
    user_ids: NotRequired[Union[
        'discord_typings.Snowflake',
        List['discord_typings.Snowflake'],
    ]]
    nonce: NotRequired[str]


class _UserIDsRequestMembersCommand(TypedDict):
    guild_id: 'discord_typings.Snowflake'
    presences: NotRequired[bool]
    user_ids: Union['discord_typings.Snowflake', List['discord_typings.Snowflake']]
    nonce: NotRequired[str]


# This enforces the fact that 'limit' is required when 'query' is set.
RequestGuildMembersData = Union[_QueryRequestMembersCommand, _UserIDsRequestMembersCommand]


class RequestGuildMembersCommand(TypedDict):
    op: Literal[8]
    d: RequestGuildMembersData


# https://discord.com/developers/docs/topics/gateway-events#update-voice-state


class UpdateVoiceStateData(TypedDict):
    guild_id: 'discord_typings.Snowflake'
    channel_id: Optional['discord_typings.Snowflake']
    self_mute: bool
    self_deaf: bool


class UpdateVoiceStateCommand(TypedDict):
    op: Literal[4]
    d: UpdateVoiceStateData


# https://discord.com/developers/docs/topics/gateway-events#update-presence


class UpdatePresenceData(TypedDict):
    since: Optional[int]
    activities: List['discord_typings.PartialActivityData']
    status: Literal['online', 'idle', 'dnd', 'invisible']
    afk: bool


class UpdatePresenceCommand(TypedDict):
    op: Literal[3]
    d: UpdatePresenceData


# https://discord.com/developers/docs/topics/gateway-events#hello


class HelloData(TypedDict):
    heartbeat_interval: int


class HelloEvent(TypedDict):
    op: Literal[10]
    d: HelloData
    s: None
    t: None


# https://discord.com/developers/docs/topics/gateway#heartbeat-interval


class HeartbeatACKEvent(TypedDict):
    op: Literal[11]


# https://discord.com/developers/docs/topics/gateway-events#ready


class PartialApplicationData(TypedDict):
    id: 'discord_typings.Snowflake'
    flags: int


class ReadyData(TypedDict):
    v: int
    user: 'discord_typings.UserData'
    guilds: List['discord_typings.UnavailableGuildData']
    session_id: str
    resume_gateway_url: str
    shard: NotRequired[Union[Tuple[int, int], List[int]]]
    application: PartialApplicationData


ReadyEvent = GenericDispatchEvent[Literal['READY'], ReadyData]


# https://discord.com/developers/docs/topics/gateway-events#resumed


class ResumedData(TypedDict):
    ...  # It only has an undocumented _trace field


ResumedEvent = GenericDispatchEvent[Literal['RESUMED'], ResumedData]


# https://discord.com/developers/docs/topics/gateway-events#reconnect


class ReconnectEvent(TypedDict):
    op: Literal[7]
    d: None
    s: None
    t: None


# https://discord.com/developers/docs/topics/gateway-events#invalid-session


class InvalidSessionEvent(TypedDict):
    op: Literal[9]
    d: bool
    s: None
    t: None


# https://discord.com/developers/docs/topics/gateway-events#application-command-permissions-update


# Line too long
ApplicationCommandPermissionsUpdateData: TypeAlias = (
    'discord_typings.ApplicationCommandPermissionsData'
)


ApplicationCommandPermissionsUpdateEvent = GenericDispatchEvent[
    Literal['APPLICATION_COMMAND_PERMISSIONS_UPDATE'],
    ApplicationCommandPermissionsUpdateData
]


# https://discord.com/developers/docs/topics/gateway-events#auto-moderation-rule-create


AutoModerationRuleCreateData: TypeAlias = 'discord_typings.AutoModerationRuleData'


AutoModerationRuleCreateEvent = GenericDispatchEvent[
    Literal['AUTO_MODERATION_RULE_CREATE'],
    AutoModerationRuleCreateData
]


# https://discord.com/developers/docs/topics/gateway-events#auto-moderation-rule-update


AutoModerationRuleUpdateData: TypeAlias = 'discord_typings.AutoModerationRuleData'


AutoModerationRuleUpdateEvent = GenericDispatchEvent[
    Literal['AUTO_MODERATION_RULE_UPDATE'],
    AutoModerationRuleUpdateData
]


# https://discord.com/developers/docs/topics/gateway-events#auto-moderation-rule-delete


AutoModerationRuleDeleteData: TypeAlias = 'discord_typings.AutoModerationRuleData'


AutoModerationRuleDeleteEvent = GenericDispatchEvent[
    Literal['AUTO_MODERATION_RULE_DELETE'],
    AutoModerationRuleDeleteData
]


# https://discord.com/developers/docs/topics/gateway-events#auto-moderation-action-execution


class AutoModerationActionExecutionData(TypedDict):
    guild_id: 'discord_typings.Snowflake'
    action: 'discord_typings.AutoModerationActionData'
    rule_id: 'discord_typings.Snowflake'
    rule_trigger_type: 'discord_typings.AutoModerationTriggerTypes'
    user_id: 'discord_typings.Snowflake'
    channel_id: NotRequired['discord_typings.Snowflake']
    message_id: NotRequired['discord_typings.Snowflake']
    alert_system_message_id: NotRequired['discord_typings.Snowflake']
    content: NotRequired[str]
    matched_keyword: Optional[str]
    matched_content: NotRequired[Optional[str]]


AutoModerationActionExecutionEvent = GenericDispatchEvent[
    Literal['AUTO_MODERATION_ACTION_EXECUTION'],
    AutoModerationActionExecutionData
]


# https://discord.com/developers/docs/topics/gateway-events#channels


# Utility for the events below
_GuildChannelData = Union[
    'discord_typings.TextChannelData',
    'discord_typings.NewsChannelData',
    'discord_typings.VoiceChannelData',
    'discord_typings.CategoryChannelData',
    'discord_typings.ForumChannelData',
    'discord_typings.MediaChannelData',
]


# https://discord.com/developers/docs/topics/gateway-events#channel-create


ChannelCreateData = _GuildChannelData
ChannelCreateEvent = GenericDispatchEvent[Literal['CHANNEL_CREATE'], ChannelCreateData]


# https://discord.com/developers/docs/topics/gateway-events#channel-update


ChannelUpdateData = _GuildChannelData
ChannelUpdateEvent = GenericDispatchEvent[Literal['CHANNEL_UPDATE'], ChannelUpdateData]


# https://discord.com/developers/docs/topics/gateway-events#channel-delete


ChannelDeleteData = _GuildChannelData
ChannelDeleteEvent = GenericDispatchEvent[Literal['CHANNEl_DELETE'], ChannelDeleteData]


# https://discord.com/developers/docs/topics/gateway-events#thread-create


class ThreadCreateData(discord_typings.ThreadChannelData):
    newly_created: bool  # Extra THREAD_CREATE field


ThreadCreateEvent = GenericDispatchEvent[Literal['THREAD_CREATE'], ThreadCreateData]


# https://discord.com/developers/docs/topics/gateway-events#thread-update


ThreadUpdateData: TypeAlias = 'discord_typings.ThreadChannelData'
ThreadUpdateEvent = GenericDispatchEvent[Literal['THREAD_UPDATE'], ThreadUpdateData]


# https://discord.com/developers/docs/topics/gateway-events#thread-delete


class ThreadDeleteData(TypedDict):
    id: 'discord_typings.Snowflake'
    guild_id: 'discord_typings.Snowflake'
    parent_id: 'discord_typings.Snowflake'
    type: Literal[10, 11, 12]


ThreadDeleteEvent = GenericDispatchEvent[Literal['THREAD_DELETE'], ThreadDeleteData]


# https://discord.com/developers/docs/topics/gateway-events#thread-list-sync


class ThreadListSyncData(TypedDict):
    guild_id: 'discord_typings.Snowflake'
    channel_ids: NotRequired[List['discord_typings.Snowflake']]
    threads: List['discord_typings.ThreadChannelData']
    members: List['discord_typings.ThreadMemberData']


ThreadListSyncEvent = GenericDispatchEvent[Literal['THREAD_LIST_SYNC'], ThreadListSyncData]


# https://discord.com/developers/docs/topics/gateway-events#thread-member-update


class ThreadMemberUpdateData(TypedDict):
    # 'id' and 'user_id' is only missing in the GUILD_CREATE event, so we know
    # that they are present in this event (hence the missing NotRequired
    # compared to ThreadMemberData).
    id: 'discord_typings.Snowflake'
    user_id: 'discord_typings.Snowflake'
    guild_id: 'discord_typings.Snowflake'
    member: NotRequired['discord_typings.GuildMemberData']
    join_timestamp: str
    flags: int


ThreadMemberUpdateEvent = GenericDispatchEvent[
    Literal['THREAD_MEMBER_UPDATE'], ThreadMemberUpdateData
]


# https://discord.com/developers/docs/topics/gateway-events#thread-members-update


class ThreadMembersUpdateData(TypedDict):
    id: 'discord_typings.Snowflake'
    guild_id: 'discord_typings.Snowflake'
    member_count: int
    added_members: NotRequired[List['discord_typings.ThreadMemberData']]
    removed_member_ids: NotRequired[List['discord_typings.Snowflake']]


ThreadMembersUpdateEvent = GenericDispatchEvent[
    Literal['THREAD_MEMBERS_UPDATE'], ThreadMembersUpdateData
]


# https://discord.com/developers/docs/topics/gateway-events#entitlement-create


EntitlementCreateData: TypeAlias = 'discord_typings.EntitlementData'
EntitlementCreateEvent = GenericDispatchEvent[
    Literal['ENTITLEMENT_CREATE'], EntitlementCreateData
]


# https://discord.com/developers/docs/topics/gateway-events#entitlement-update


EntitlementUpdateData: TypeAlias = 'discord_typings.EntitlementData'
EntitlementUpdateEvent = GenericDispatchEvent[
    Literal['ENTITLEMENT_UPDATE'], EntitlementUpdateData
]


# https://discord.com/developers/docs/topics/gateway-events#entitlement-delete


EntitlementDeleteData: TypeAlias = 'discord_typings.EntitlementData'
EntitlementDeleteEvent = GenericDispatchEvent[
    Literal['ENTITLEMENT_DELETE'], EntitlementDeleteData
]


# https://discord.com/developers/docs/topics/gateway-events#channel-pins-update


class ChannelPinsUpdateData(TypedDict):
    guild_id: NotRequired['discord_typings.Snowflake']
    channel_id: 'discord_typings.Snowflake'
    last_pin_timestamp: NotRequired[Optional[str]]


ChannelPinsUpdateEvent = GenericDispatchEvent[
    Literal['CHANNEL_PINS_UPDATE'], ChannelPinsUpdateData
]


# https://discord.com/developers/docs/topics/gateway-events#guild-create


class GuildCreateData(discord_typings.GuildData):
    # Extra GUILD_CREATE fields
    joined_at: str
    large: bool
    unavailable: bool
    member_count: int
    voice_states: List['discord_typings.VoiceStateData']
    members: List['discord_typings.GuildMemberData']
    channels: List[_GuildChannelData]  # Alias from above
    threads: List['discord_typings.ThreadChannelData']
    presences: List['discord_typings.UpdatePresenceData']
    stage_instances: List['discord_typings.StageInstanceData']
    guild_scheduled_events: List['discord_typings.GuildScheduledEventData']


GuildCreateEvent = GenericDispatchEvent[Literal['GUILD_CREATE'], GuildCreateData]


# https://discord.com/developers/docs/topics/gateway-events#guild-update


GuildUpdateData: TypeAlias = 'discord_typings.GuildData'
GuildUpdateEvent = GenericDispatchEvent[Literal['GUILD_UPDATE'], GuildUpdateData]


# https://discord.com/developers/docs/topics/gateway-events#guild-delete


GuildDeleteData: TypeAlias = 'discord_typings.UnavailableGuildData'
GuildDeleteEvent = GenericDispatchEvent[Literal['GUILD_DELETE'], 'GuildDeleteData']


# https://discord.com/developers/docs/topics/gateway-events#guild-audit-log-entry-create


GuildAuditLogEntryCreateData: TypeAlias = 'discord_typings.AuditLogEntryData'
GuildAuditLogEntryCreateEvent = GenericDispatchEvent[
    Literal['GUILD_AUDIT_LOG_ENTRY_CREATE'],
    GuildAuditLogEntryCreateData
]


# https://discord.com/developers/docs/topics/gateway-events#guild-ban-add


class GuildBanAddData(TypedDict):
    guild_id: 'discord_typings.Snowflake'
    user: 'discord_typings.UserData'


GuildBanAddEvent = GenericDispatchEvent[Literal['GUILD_BAN_ADD'], GuildBanAddData]


# https://discord.com/developers/docs/topics/gateway-events#guild-ban-remove


class GuildBanRemoveData(TypedDict):
    guild_id: 'discord_typings.Snowflake'
    user: 'discord_typings.UserData'


GuildBanRemoveEvent = GenericDispatchEvent[Literal['GUILD_BAN_REMOVE'], GuildBanRemoveData]


# https://discord.com/developers/docs/topics/gateway-events#guild-emojis-update


class GuildEmojisUpdateData(TypedDict):
    guild_id: 'discord_typings.Snowflake'
    emojis: List['discord_typings.EmojiData']


GuildEmojisUpdateEvent = GenericDispatchEvent[
    Literal['GUILD_EMOJIS_UPDATE'], GuildEmojisUpdateData
]


# https://discord.com/developers/docs/topics/gateway-events#guild-stickers-update


class GuildStickersUpdateData(TypedDict):
    guild_id: 'discord_typings.Snowflake'
    stickers: List['discord_typings.StickerData']


GuildStickersUpdateEvent = GenericDispatchEvent[
    Literal['GUILD_STICKERS_UPDATE'], GuildStickersUpdateData
]


# https://discord.com/developers/docs/topics/gateway-events#guild-integrations-update


class GuildIntergrationsUpdateData(TypedDict):
    guild_id: 'discord_typings.Snowflake'


GuildIntergrationsUpdateEvent = GenericDispatchEvent[
    Literal['GUILD_INTEGRATIONS_UPDATE'], GuildIntergrationsUpdateData
]


# https://discord.com/developers/docs/topics/gateway-events#guild-member-add


class GuildMemberAddData(TypedDict):
    user: 'discord_typings.UserData'  # Always present in GUILD_MEMBER_ADD events
    nick: NotRequired[Optional[str]]
    avatar: NotRequired[Optional[str]]
    roles: List['discord_typings.Snowflake']
    joined_at: str
    premium_since: NotRequired[Optional[str]]
    deaf: bool
    mute: bool
    pending: NotRequired[bool]
    permissions: NotRequired[str]
    communication_disabled_until: NotRequired[Optional[str]]

    guild_id: 'discord_typings.Snowflake'  # Extra field


GuildMemberAddEvent = GenericDispatchEvent[
    Literal['GUILD_MEMBER_ADD'], GuildMemberAddData
]


# https://discord.com/developers/docs/topics/gateway-events#guild-member-remove


class GuildMemberRemoveData(TypedDict):
    user: 'discord_typings.UserData'
    guild_id: 'discord_typings.Snowflake'


GuildMemberRemoveEvent = GenericDispatchEvent[
    Literal['GUILD_MEMBER_REMOVE'], GuildMemberRemoveData
]


# https://discord.com/developers/docs/topics/gateway-events#guild-member-update


class GuildMemberUpdateData(TypedDict):
    guild_id: 'discord_typings.Snowflake'
    roles: List['discord_typings.Snowflake']
    user: 'discord_typings.UserData'
    nick: NotRequired[Optional[str]]
    avatar: Optional[str]
    joined_at: Optional[str]
    premium_since: NotRequired[Optional[str]]
    deaf: NotRequired[bool]
    mute: NotRequired[bool]
    pending: NotRequired[bool]
    communication_disabled_until: NotRequired[Optional[str]]


GuildMemberUpdateEvent = GenericDispatchEvent[
    Literal['GUILD_MEMBER_UPDATE'], GuildMemberUpdateData
]


# https://discord.com/developers/docs/topics/gateway-events#guild-members-chunk


class GuildMembersChunkData(TypedDict):
    guild_id: 'discord_typings.Snowflake'
    members: List['discord_typings.GuildMemberData']
    chunk_index: int
    chunk_count: int
    not_found: NotRequired[List['discord_typings.Snowflake']]
    presences: NotRequired[List['discord_typings.PresenceUpdateData']]
    nonce: NotRequired[str]


GuildMembersChunkEvent = GenericDispatchEvent[
    Literal['GUILD_MEMBERS_CHUNK'], GuildMembersChunkData
]


# https://discord.com/developers/docs/topics/gateway-events#guild-role-create


class GuildRoleCreateData(TypedDict):
    guild_id: 'discord_typings.Snowflake'
    role: 'discord_typings.RoleData'


GuildRoleCreateEvent = GenericDispatchEvent[Literal['GUILD_ROLE_CREATE'], GuildRoleCreateData]


# https://discord.com/developers/docs/topics/gateway-events#guild-role-update


class GuildRoleUpdateData(TypedDict):
    guild_id: 'discord_typings.Snowflake'
    role: 'discord_typings.RoleData'


GuildRoleUpdateEvent = GenericDispatchEvent[Literal['GUILD_ROLE_UPDATE'], GuildRoleUpdateData]


# https://discord.com/developers/docs/topics/gateway-events#guild-role-delete


class GuildRoleDeleteData(TypedDict):
    guild_id: 'discord_typings.Snowflake'
    role_id: 'discord_typings.Snowflake'


GuildRoleDeleteEvent = GenericDispatchEvent[Literal['GUILD_ROLE_DELETE'], GuildRoleDeleteData]


# https://discord.com/developers/docs/topics/gateway-events#guild-scheduled-event-create


GuildScheduledEventCreateData: TypeAlias = 'discord_typings.GuildScheduledEventData'
GuildScheduledEventCreateEvent = GenericDispatchEvent[
    Literal['GUILD_SCHEDULED_EVENT_CREATE'], GuildScheduledEventCreateData
]


# https://discord.com/developers/docs/topics/gateway-events#guild-scheduled-event-update


GuildScheduledEventUpdateData: TypeAlias = 'discord_typings.GuildScheduledEventData'
GuildScheduledEventUpdateEvent = GenericDispatchEvent[
    Literal['GUILD_SCHEDULED_EVENT_UPDATE'], GuildScheduledEventUpdateData
]


# https://discord.com/developers/docs/topics/gateway-events#guild-scheduled-event-delete


GuildScheduledEventDeleteData: TypeAlias = 'discord_typings.GuildScheduledEventData'
GuildScheduledEventDeleteEvent = GenericDispatchEvent[
    Literal['GUILD_SCHEDULED_EVENT_DELETE'], GuildScheduledEventDeleteData
]


# https://discord.com/developers/docs/topics/gateway-events#guild-scheduled-event-user-add


class GuildScheduledEventUserAddData(TypedDict):
    guild_scheduled_event_id: 'discord_typings.Snowflake'
    user_id: 'discord_typings.Snowflake'
    guild_id: 'discord_typings.Snowflake'


GuildScheduledEventUserAddEvent = GenericDispatchEvent[
    Literal['GUILD_SCHEDULED_EVENT_USER_ADD'], GuildScheduledEventUserAddData
]


# https://discord.com/developers/docs/topics/gateway-events#guild-scheduled-event-user-remove


class GuildScheduledEventUserRemoveData(TypedDict):
    guild_scheduled_event_id: 'discord_typings.Snowflake'
    user_id: 'discord_typings.Snowflake'
    guild_id: 'discord_typings.Snowflake'


GuildScheduledEventUserRemoveEvent = GenericDispatchEvent[
    Literal['GUILD_SCHEDULED_EVENT_USER_REMOVE'], GuildScheduledEventUserRemoveData
]


# https://discord.com/developers/docs/topics/gateway-events#integrations


class _StreamingIntegrationGuildData(TypedDict):
    id: 'discord_typings.Snowflake'
    name: str
    type: Literal['twitch', 'youtube', 'discord']
    enabled: bool
    syncing: bool
    role_id: NotRequired['discord_typings.Snowflake']
    enable_emoticons: bool
    expire_behavior: 'discord_typings.IntegrationExpireBehaviors'
    expire_grace_period: int
    user: NotRequired['discord_typings.UserData']
    account: 'discord_typings.IntegrationAccountData'
    synced_at: str
    subscriber_count: int
    revoked: bool
    application: 'discord_typings.IntegrationApplicationData'
    scopes: NotRequired[List['discord_typings.OAuth2Scopes']]

    guild_id: 'discord_typings.Snowflake'


class _DiscordIntegrationGuildData(TypedDict):
    id: 'discord_typings.Snowflake'
    name: str
    type: Literal['discord']
    enabled: bool
    user: NotRequired['discord_typings.UserData']
    account: 'discord_typings.IntegrationAccountData'
    application: NotRequired['discord_typings.IntegrationApplicationData']
    scopes: NotRequired[List['discord_typings.OAuth2Scopes']]

    guild_id: 'discord_typings.Snowflake'


# Integration object, with an additional guild_id field
_IntegrationGuildData = Union[_StreamingIntegrationGuildData, _DiscordIntegrationGuildData]

# https://discord.com/developers/docs/topics/gateway-events#integration-create


IntegrationCreateData: TypeAlias = '_IntegrationGuildData'
IntegrationCreateEvent = GenericDispatchEvent[
    Literal['INTEGRATION_CREATE'], IntegrationCreateData
]


# https://discord.com/developers/docs/topics/gateway-events#integration-update


IntegrationUpdateData: TypeAlias = '_IntegrationGuildData'
IntegrationUpdateEvent = GenericDispatchEvent[
    Literal['INTEGRATION_UPDATE'], IntegrationUpdateData
]


# https://discord.com/developers/docs/topics/gateway-events#integration-delete


class IntegrationDeleteData(TypedDict):
    id: 'discord_typings.Snowflake'
    guild_id: 'discord_typings.Snowflake'
    application_id: NotRequired['discord_typings.Snowflake']


IntegrationDeleteEvent = GenericDispatchEvent[
    Literal['INTEGRATION_DELETE'], IntegrationDeleteData
]


# https://discord.com/developers/docs/topics/gateway-events#invite-create


class InviteCreateData(TypedDict):
    channel_id: 'discord_typings.Snowflake'
    code: str
    created_at: str
    guild_id: NotRequired['discord_typings.Snowflake']
    inviter: NotRequired['discord_typings.UserData']
    max_age: int
    max_uses: int
    target_type: NotRequired[Literal[1, 2]]
    target_user: NotRequired['discord_typings.UserData']
    target_application: NotRequired['discord_typings.ApplicationData']
    temporary: bool
    uses: int


InviteCreateEvent = GenericDispatchEvent[Literal['INVITE_CREATE'], InviteCreateData]


# https://discord.com/developers/docs/topics/gateway-events#invite-delete


class InviteDeleteData(TypedDict):
    channel_id: 'discord_typings.Snowflake'
    guild_id: NotRequired['discord_typings.Snowflake']
    code: str


InviteDeleteEvent = GenericDispatchEvent[Literal['INVITE_DELETE'], InviteDeleteData]


# https://discord.com/developers/docs/topics/gateway-events#message-create


MessageCreateData: TypeAlias = 'discord_typings.MessageData'
MessageCreateEvent = GenericDispatchEvent[Literal['MESSAGE_CREATE'], MessageCreateData]


# https://discord.com/developers/docs/topics/gateway-events#message-update


MessageUpdateData: TypeAlias = 'discord_typings.MessageData'
MessageUpdateEvent = GenericDispatchEvent[Literal['MESSAGE_UPDATE'], MessageUpdateData]


# https://discord.com/developers/docs/topics/gateway-events#message-delete


class MessageDeleteData(TypedDict):
    id: 'discord_typings.Snowflake'
    channel_id: 'discord_typings.Snowflake'
    guild_id: NotRequired['discord_typings.Snowflake']


MessageDeleteEvent = GenericDispatchEvent[Literal['MESSAGE_DELETE'], MessageDeleteData]


# https://discord.com/developers/docs/topics/gateway-events#message-delete-bulk


class MessageDeleteBulkData(TypedDict):
    ids: List['discord_typings.Snowflake']
    channel_id: 'discord_typings.Snowflake'
    guild_id: NotRequired['discord_typings.Snowflake']


MessageDeleteBulkEvent = GenericDispatchEvent[
    Literal['MESSAGE_DELETE_BULK'], MessageDeleteBulkData
]


# https://discord.com/developers/docs/topics/gateway-events#message-reaction-add


class MessageReactionAddData(TypedDict):
    user_id: 'discord_typings.Snowflake'
    channel_id: 'discord_typings.Snowflake'
    message_id: 'discord_typings.Snowflake'
    guild_id: NotRequired['discord_typings.Snowflake']
    member: NotRequired['discord_typings.GuildMemberData']
    emoji: 'discord_typings.EmojiData'
    message_author_id: NotRequired['discord_typings.Snowflake']


MessageReactionAddEvent = GenericDispatchEvent[
    Literal['MESSAGE_REACTION_ADD'], MessageReactionAddData
]


# https://discord.com/developers/docs/topics/gateway-events#message-reaction-remove


class MessageReactionRemoveData(TypedDict):
    user_id: 'discord_typings.Snowflake'
    channel_id: 'discord_typings.Snowflake'
    message_id: 'discord_typings.Snowflake'
    guild_id: NotRequired['discord_typings.Snowflake']
    emoji: 'discord_typings.EmojiData'


MessageReactionRemoveEvent = GenericDispatchEvent[
    Literal['MESSAGE_REACTION_REMOVE'], MessageReactionRemoveData
]


# https://discord.com/developers/docs/topics/gateway-events#message-reaction-remove-all


class MessageReactionRemoveAllData(TypedDict):
    channel_id: 'discord_typings.Snowflake'
    message_id: 'discord_typings.Snowflake'
    guild_id: NotRequired['discord_typings.Snowflake']


MessageReactionRemoveAllEvent = GenericDispatchEvent[
    Literal['MESSAGE_REACTION_REMOVE_ALL'], MessageReactionRemoveAllData
]


# https://discord.com/developers/docs/topics/gateway-events#message-reaction-remove-emoji


class MessageReactionRemoveEmojiData(TypedDict):
    channel_id: 'discord_typings.Snowflake'
    guild_id: NotRequired['discord_typings.Snowflake']
    message_id: 'discord_typings.Snowflake'
    emoji: 'discord_typings.EmojiData'


MessageReactionRemoveEmojiEvent = GenericDispatchEvent[
    Literal['MESSAGE_REACTION_REMOVE_EMOJI'], MessageReactionRemoveEmojiData
]


# https://discord.com/developers/docs/topics/gateway-events#presence-update


class PresenceUpdateData(TypedDict):
    user: 'discord_typings.UserData'
    guild_id: 'discord_typings.Snowflake'
    status: Literal['idle', 'dnd', 'online', 'offline']
    activities: List['discord_typings.ActivityData']
    client_status: 'discord_typings.ClientStatusData'


class ClientStatusData(TypedDict):
    desktop: NotRequired[str]
    mobile: NotRequired[str]
    web: NotRequired[str]


# https://discord.com/developers/docs/topics/gateway-events#activity-object


class PartialActivityData(TypedDict):
    name: str
    type: 'discord_typings.ActivityTypes'
    state: NotRequired[Optional[str]]
    url: NotRequired[str]


class ActivityData(TypedDict):
    name: str
    type: Literal[0, 1, 2, 3, 4, 5]
    url: NotRequired[Optional[str]]
    created_at: int
    timestamps: NotRequired['discord_typings.ActivityTimestampsData']
    application_id: NotRequired['discord_typings.Snowflake']
    details: NotRequired[Optional[str]]
    state: NotRequired[Optional[str]]
    emoji: NotRequired[Optional['discord_typings.ActivityEmojiData']]
    party: NotRequired[Optional['discord_typings.ActivityPartyData']]
    assets: NotRequired['discord_typings.ActivityAssetsData']
    secrets: NotRequired['discord_typings.ActivitySecretsData']
    instance: NotRequired[bool]
    flags: NotRequired[int]
    buttons: NotRequired[List['discord_typings.ActivityButtonData']]


# https://discord.com/developers/docs/topics/gateway-events#activity-object-activity-types


ActivityTypes = Literal[0, 1, 2, 3, 4, 5]


# https://discord.com/developers/docs/topics/gateway-events#activity-object-activity-timestamps


class ActivityTimestampsData(TypedDict):
    start: NotRequired[int]
    end: NotRequired[int]


# https://discord.com/developers/docs/topics/gateway-events#activity-object-activity-emoji


class ActivityEmojiData(TypedDict):
    name: str
    id: NotRequired['discord_typings.Snowflake']
    animated: NotRequired[bool]


# https://discord.com/developers/docs/topics/gateway-events#activity-object-activity-party


class ActivityPartyData(TypedDict):
    id: NotRequired[str]
    size: NotRequired[Union[Tuple[int, int], List[int]]]


# https://discord.com/developers/docs/topics/gateway-events#activity-object-activity-assets


class ActivityAssetsData(TypedDict):
    large_image: NotRequired[str]
    large_text: NotRequired[str]
    small_image: NotRequired[str]
    small_test: NotRequired[str]


# https://discord.com/developers/docs/topics/gateway-events#activity-object-activity-secrets


class ActivitySecretsData(TypedDict):
    join: NotRequired[str]
    spectate: NotRequired[str]
    match: NotRequired[str]


# https://discord.com/developers/docs/topics/gateway-events#activity-object-activity-buttons


class ActivityButtonData(TypedDict):
    label: str
    url: str


# https://discord.com/developers/docs/topics/gateway-events#typing-start


class TypingStartData(TypedDict):
    channel_id: 'discord_typings.Snowflake'
    guild_id: NotRequired['discord_typings.Snowflake']
    user_id: 'discord_typings.Snowflake'
    timestamp: int
    member: NotRequired['discord_typings.GuildMemberData']


TypingStartEvent = GenericDispatchEvent[Literal['TYPING_START'], TypingStartData]


# https://discord.com/developers/docs/topics/gateway-events#user-update


UserUpdateData: TypeAlias = 'discord_typings.UserData'
UserUpdateEvent = GenericDispatchEvent[Literal['USER_UPDATE'], UserUpdateData]


# https://discord.com/developers/docs/topics/gateway-events#voice-state-update


VoiceStateUpdateData: TypeAlias = 'discord_typings.VoiceStateData'
VoiceStateUpdateEvent = GenericDispatchEvent[
    Literal['VOICE_STATUS_UPDATE'], VoiceStateUpdateData
]


# https://discord.com/developers/docs/topics/gateway-events#voice-server-update


class VoiceServerUpdateData(TypedDict):
    token: str
    guild_id: 'discord_typings.Snowflake'
    endpoint: Optional[str]


VoiceServerUpdateEvent = GenericDispatchEvent[
    Literal['VOICE_SERVER_UPDATE'], VoiceServerUpdateData
]


# https://discord.com/developers/docs/topics/gateway-events#webhooks-update


class WebhooksUpdateData(TypedDict):
    guild_id: 'discord_typings.Snowflake'
    channel_id: 'discord_typings.Snowflake'


WebhooksUpdateEvent = GenericDispatchEvent[Literal['WEBHOOKS_UPDATE'], WebhooksUpdateData]


# https://discord.com/developers/docs/topics/gateway-events#interaction-create


InteractionCreateData: TypeAlias = 'discord_typings.InteractionData'
InteractionCreateEvent = GenericDispatchEvent[
    Literal['INTERACTION_CREATE'], InteractionCreateData
]


# https://discord.com/developers/docs/topics/gateway-events#stage-instance-create


StageInstanceCreateData: TypeAlias = 'discord_typings.StageInstanceData'
StageInstanceCreateEvent = GenericDispatchEvent[
    Literal['STAGE_INSTANCE_CREATE'], StageInstanceCreateData
]


# https://discord.com/developers/docs/topics/gateway-events#stage-instance-update


StageInstanceUpdateData: TypeAlias = 'discord_typings.StageInstanceData'
StageInstanceUpdateEvent = GenericDispatchEvent[
    Literal['STAGE_INSTANCE_UPDATE'], StageInstanceUpdateData
]


# https://discord.com/developers/docs/topics/gateway-events#stage-instance-delete


StageInstanceDeleteData: TypeAlias = 'discord_typings.StageInstanceData'
StageInstanceDeleteEvent = GenericDispatchEvent[
    Literal['STAGE_INSTANCE_DELETE'], StageInstanceDeleteData
]


# Generalized unions for the typings in this file


DispatchEvent = GenericDispatchEvent[str, Dict[str, Any]]


GatewayCommand = Union[
    IdentifyCommand,
    ResumeCommand,
    HeartbeatCommand,
    RequestGuildMembersCommand,
    UpdateVoiceStateCommand,
    UpdatePresenceCommand,
]


GatewayEvent = Union[
    HelloEvent,
    HeartbeatACKEvent,
    ReadyEvent,
    ResumedEvent,
    ReconnectEvent,
    InvalidSessionEvent,
    DispatchEvent,
]



from typing import List, Optional

from typing_extensions import Literal, NotRequired, TypedDict

import discord_typings

__all__ = (
    'StickerItemData',
    'StickerTypes',
    'StickerFormatTypes',
    'StickerData',
    'StickerPackData',
)


# https://discord.com/developers/docs/resources/sticker#sticker-item-object


# Because of inheritance, this class comes first even though that is not the
# order in the documentation.
class StickerItemData(TypedDict):
    id: 'discord_typings.Snowflake'
    name: str
    format_type: 'discord_typings.StickerFormatTypes'


# https://discord.com/developers/docs/resources/sticker#sticker-object-sticker-types


StickerTypes = Literal[1, 2]


# https://discord.com/developers/docs/resources/sticker#sticker-object-sticker-format-types


StickerFormatTypes = Literal[1, 2, 3, 4]


# https://discord.com/developers/docs/resources/sticker#sticker-object-sticker-structure


class StickerData(StickerItemData):
    pack_id: NotRequired['discord_typings.Snowflake']
    description: Optional[str]
    tags: str
    type: StickerTypes
    available: NotRequired[bool]
    guild_id: NotRequired['discord_typings.Snowflake']
    user: NotRequired['discord_typings.UserData']
    sort_value: NotRequired[int]


# https://discord.com/developers/docs/resources/sticker#sticker-pack-object


class StickerPackData(TypedDict):
    id: 'discord_typings.Snowflake'
    stickers: List['discord_typings.StickerData']
    name: str
    sku_id: 'discord_typings.Snowflake'
    cover_sticker_id: NotRequired['discord_typings.Snowflake']
    description: str
    banner_asset_id: NotRequired['discord_typings.Snowflake']



from typing import Optional, Union

from typing_extensions import Literal, NotRequired, TypedDict

import discord_typings

__all__ = (
    'StageGuildScheduledEventData',
    'VoiceGuildScheduledEventData',
    'ExternalGuildScheduledEventData',
    'GuildScheduledEventData',
    'GuildScheduledEventPrivacyLevels',
    'GuildScheduledEventStatus',
    'GuildScheduledEventEntityTypes',
    'GuildScheduledEventEntityMetadataData',
    'GuildScheduledEventUserData',
)


# https://discord.com/developers/docs/resources/guild-scheduled-event#guild-scheduled-event-object-guild-scheduled-event-structure


class _GuildScheduledEventData(TypedDict):
    id: 'discord_typings.Snowflake'
    guild_id: 'discord_typings.Snowflake'
    creator_id: 'discord_typings.Snowflake'
    name: str
    description: NotRequired[Optional[str]]
    scheduled_start_time: str
    privacy_level: 'discord_typings.GuildScheduledEventPrivacyLevels'
    status: 'discord_typings.GuildScheduledEventStatus'
    entity_id: Optional['discord_typings.Snowflake']
    creator: 'discord_typings.UserData'
    user_count: NotRequired[int]
    image: NotRequired[Optional[str]]


class StageGuildScheduledEventData(_GuildScheduledEventData):
    entity_type: Literal[1]
    channel_id: 'discord_typings.Snowflake'
    entity_metadata: None
    scheduled_end_time: NotRequired[str]


class VoiceGuildScheduledEventData(_GuildScheduledEventData):
    entity_type: Literal[2]
    channel_id: 'discord_typings.Snowflake'
    entity_metadata: None
    scheduled_end_time: NotRequired[str]


class ExternalGuildScheduledEventData(_GuildScheduledEventData):
    entity_type: Literal[3]
    channel_id: None
    entity_metadata: 'discord_typings.GuildScheduledEventEntityMetadataData'
    scheduled_end_time: str


GuildScheduledEventData = Union[
    StageGuildScheduledEventData,
    VoiceGuildScheduledEventData,
    ExternalGuildScheduledEventData
]


# https://discord.com/developers/docs/resources/guild-scheduled-event#guild-scheduled-event-object-guild-scheduled-event-privacy-level


GuildScheduledEventPrivacyLevels = Literal[2]


# https://discord.com/developers/docs/resources/guild-scheduled-event#guild-scheduled-event-object-guild-scheduled-event-entity-types


GuildScheduledEventEntityTypes = Literal[1, 2, 3]


# https://discord.com/developers/docs/resources/guild-scheduled-event#guild-scheduled-event-object-guild-scheduled-event-status


GuildScheduledEventStatus = Literal[1, 2, 3, 4]


# https://discord.com/developers/docs/resources/guild-scheduled-event#guild-scheduled-event-object-guild-scheduled-event-entity-metadata


class GuildScheduledEventEntityMetadataData(TypedDict):
    location: NotRequired[str]


# https://discord.com/developers/docs/resources/guild-scheduled-event#guild-scheduled-event-user-object


class GuildScheduledEventUserData(TypedDict):
    guild_scheduled_event_id: 'discord_typings.Snowflake'
    user: 'discord_typings.UserData'
    member: NotRequired['discord_typings.GuildMemberData']



from typing import Any, List, Optional

from typing_extensions import Literal, NotRequired, TypedDict

import discord_typings

__all__ = (
    'AuditLogData',
    'PartialIntegrationData',
    'AuditLogEntryData',
    'OptionalAuditLogEntryData',
    'AuditLogChangeData',
    'AuditLogEvents',
)


# https://discord.com/developers/docs/resources/audit-log#audit-log-object-audit-log-structure


class AuditLogData(TypedDict):
    application_commands: List['discord_typings.ApplicationCommandData']
    audit_log_entries: List['discord_typings.AuditLogEntryData']
    auto_moderation_rules: List['discord_typings.AutoModerationRuleData']
    guild_scheduled_events: List['discord_typings.GuildScheduledEventData']
    integrations: List['discord_typings.PartialIntegrationData']
    threads: List['discord_typings.ThreadChannelData']
    users: List['discord_typings.UserData']
    webhooks: List['discord_typings.WebhookData']


class PartialIntegrationData(TypedDict):
    id: 'discord_typings.Snowflake'
    name: str
    type: Literal['twitch', 'youtube', 'discord']
    account: 'discord_typings.IntegrationAccountData'
    application_id: 'discord_typings.Snowflake'


# https://discord.com/developers/docs/resources/audit-log#audit-log-entry-object-audit-log-entry-structure


class AuditLogEntryData(TypedDict):
    target_id: Optional[str]
    changes: NotRequired[List['discord_typings.AuditLogChangeData']]
    user_id: Optional['discord_typings.Snowflake']
    id: 'discord_typings.Snowflake'
    action_type: 'discord_typings.AuditLogEvents'
    options: NotRequired['discord_typings.OptionalAuditLogEntryData']
    reason: NotRequired[str]


# https://discord.com/developers/docs/resources/audit-log#audit-log-entry-object-audit-log-events


AuditLogEvents = Literal[
    1,
    10, 11, 12, 13, 14, 15,
    20, 21, 22, 23, 24, 25, 26, 27, 28,
    30, 31, 32,
    40, 41, 42,
    50, 51, 52,
    60, 61, 62,
    72, 73, 74, 75,
    80, 81, 82, 83, 84, 85,
    90, 91, 92,
    100, 101, 102,
    110, 111, 112,
    121,
    140, 141, 142, 143, 144, 145,
    150, 151
]


# https://discord.com/developers/docs/resources/audit-log#audit-log-entry-object-optional-audit-entry-info


class OptionalAuditLogEntryData(TypedDict):
    application_id: NotRequired['discord_typings.Snowflake']
    auto_moderation_rule_name: NotRequired[str]
    auto_moderation_trigger_type: NotRequired[str]  # AutoModerationTriggerTypes
    channel_id: NotRequired['discord_typings.Snowflake']
    count: NotRequired[str]
    delete_member_days: NotRequired[str]
    id: NotRequired['discord_typings.Snowflake']
    members_removed: NotRequired[str]
    message_id: NotRequired['discord_typings.Snowflake']
    role_name: NotRequired[str]
    type: NotRequired[Literal['0', '1']]
    integration_type: str


# https://discord.com/developers/docs/resources/audit-log#audit-log-change-object-audit-log-change-structure


class AuditLogChangeData(TypedDict):
    new_value: NotRequired[Any]
    old_value: NotRequired[Any]
    key: str



from typing import List, Optional

from typing_extensions import NotRequired, TypedDict

import discord_typings

__all__ = ('EmojiData',)


# https://discord.com/developers/docs/resources/emoji#emoji-object


class EmojiData(TypedDict):
    id: Optional['discord_typings.Snowflake']
    name: Optional[str]
    roles: NotRequired[List['discord_typings.Snowflake']]
    user: NotRequired['discord_typings.UserData']
    require_colons: NotRequired[bool]
    managed: NotRequired[bool]
    animated: NotRequired[bool]
    available: NotRequired[bool]



from typing import List

from typing_extensions import Literal, NotRequired, TypedDict

import discord_typings

__all__ = (
    'OAuth2Scopes',
    'AccessTokenResponseData',
    'AuthorizationInformationData',
)


# https://discord.com/developers/docs/topics/oauth2#shared-resources-oauth2-scopes


OAuth2Scopes = Literal[
    'activities.read',
    'activities.write',
    'applications.builds.read',
    'applications.build.upload',
    'applications.commands',
    'applications.commands.update',
    'applications.commands.permissions.update',
    'application.entitlements',
    'applications.store.update',
    'bot',
    'connections',
    'dm_channels.read',
    'email',
    'gdm.join',
    'guilds',
    'guilds.join',
    'guilds.members.read',
    'identify',
    'messages.read',
    'relationships.read',
    'role_connections.write',
    'rpc',
    'rpc.activities.write',
    'rpc.notifications.read',
    'rpc.voice.read',
    'rpc.voice.write',
    'voice',
    'webhook.incoming',
]


# https://discord.com/developers/docs/topics/oauth2#authorization-code-grant-access-token-response


class AccessTokenResponseData(TypedDict):
    access_token: str
    token_type: str
    expires_in: int
    refresh_token: str
    scope: str


# https://discord.com/developers/docs/topics/oauth2#get-current-authorization-information


class AuthorizationInformationData(TypedDict):
    application: 'discord_typings.ApplicationData'
    scopes: List[str]
    user: NotRequired['discord_typings.UserData']
    expires: str



from typing import List, Union

from typing_extensions import Literal, NotRequired, TypedDict

import discord_typings

__all__ = (
    'KeywordAutoModerationRuleData',
    'SpamAutoModerationRuleData',
    'KeywordPresetAutoModerationRuleData',
    'MentionSpamAutoModerationRuleData',
    'AutoModerationRuleData',
    'AutoModerationTriggerTypes',
    'KeywordAutoModerationTriggerMetadata',
    'KeywordPresetAutoModerationTriggerMetadataData',
    'MentionSpamAutoModerationTriggerMetadataData',
    'EmptyAutoModerationTriggerMetadataData',
    'AutoModerationTriggerMetadataData',
    'AutoModerationKeywordPresetTypes',
    'AutoModerationEventTypes',
    'BlockMessageAutoModerationActionData',
    'SendAlertMessageAutoModerationActionData',
    'TimeoutAutoModerationActionData',
    'AutoModerationActionData',
    'AutoModerationActionTypes',
    'BlockmessageAutoModerationActionMetadataData',
    'SendAlertMessageAutoModerationActionMetadataData',
    'TimeoutAutoModerationActionMetadataData',
    'AutoModerationActionMetadataData',
)


# https://discord.com/developers/docs/resources/auto-moderation#auto-moderation-rule-object-auto-moderation-rule-structure


class _AutoModerationRuleData(TypedDict):
    id: 'discord_typings.Snowflake'
    guild_id: 'discord_typings.Snowflake'
    name: str
    creator_id: 'discord_typings.Snowflake'
    event_type: 'discord_typings.AutoModerationEventTypes'
    actions: List['discord_typings.AutoModerationActionData']
    enabled: bool
    exempt_roles: List['discord_typings.Snowflake']
    exempt_channels: List['discord_typings.Snowflake']


class KeywordAutoModerationRuleData(_AutoModerationRuleData):
    trigger_type: Literal[1]
    trigger_metadata: 'discord_typings.KeywordAutoModerationTriggerMetadata'


class SpamAutoModerationRuleData(_AutoModerationRuleData):
    trigger_type: Literal[3]
    trigger_metadata: 'discord_typings.EmptyAutoModerationTriggerMetadataData'


class KeywordPresetAutoModerationRuleData(_AutoModerationRuleData):
    trigger_type: Literal[4]
    trigger_metadata: 'discord_typings.KeywordPresetAutoModerationTriggerMetadataData'


class MentionSpamAutoModerationRuleData(_AutoModerationRuleData):
    trigger_type: Literal[5]
    trigger_metadata: 'discord_typings.MentionSpamAutoModerationTriggerMetadataData'


AutoModerationRuleData = Union[
    KeywordAutoModerationRuleData, KeywordPresetAutoModerationRuleData,
    SpamAutoModerationRuleData, MentionSpamAutoModerationRuleData
]


# https://discord.com/developers/docs/resources/auto-moderation#auto-moderation-rule-object-trigger-types


AutoModerationTriggerTypes = Literal[1, 2, 3, 4]


# https://discord.com/developers/docs/resources/auto-moderation#auto-moderation-rule-object-trigger-metadata


class KeywordAutoModerationTriggerMetadata(TypedDict):
    keyword_filter: List[str]
    regex_patterns: List[str]

    allow_list: List[str]


class KeywordPresetAutoModerationTriggerMetadataData(TypedDict):
    presets: List['discord_typings.AutoModerationKeywordPresetTypes']
    allow_list: List[str]


class MentionSpamAutoModerationTriggerMetadataData(TypedDict):
    mention_total_limit: int
    mention_raid_protection_enabled: bool


class EmptyAutoModerationTriggerMetadataData(TypedDict):
    ...


AutoModerationTriggerMetadataData = Union[
    KeywordAutoModerationTriggerMetadata,
    KeywordPresetAutoModerationTriggerMetadataData,
    MentionSpamAutoModerationTriggerMetadataData,
    EmptyAutoModerationTriggerMetadataData,
]


# https://discord.com/developers/docs/resources/auto-moderation#auto-moderation-rule-object-keyword-preset-types


AutoModerationKeywordPresetTypes = Literal[1, 2, 3]


# https://discord.com/developers/docs/resources/auto-moderation#auto-moderation-rule-object-event-types


AutoModerationEventTypes = Literal[1]


# https://discord.com/developers/docs/resources/auto-moderation#auto-moderation-action-object-auto-moderation-action-structure


class BlockMessageAutoModerationActionData(TypedDict):
    type: Literal[1]
    metadata: NotRequired['discord_typings.BlockmessageAutoModerationActionMetadataData']


class SendAlertMessageAutoModerationActionData(TypedDict):
    type: Literal[2]
    metadata: 'discord_typings.SendAlertMessageAutoModerationActionMetadataData'


class TimeoutAutoModerationActionData(TypedDict):
    type: Literal[3]
    metadata: 'discord_typings.TimeoutAutoModerationActionMetadataData'


AutoModerationActionData = Union[
    BlockMessageAutoModerationActionData, SendAlertMessageAutoModerationActionData,
    TimeoutAutoModerationActionData
]


# https://discord.com/developers/docs/resources/auto-moderation#auto-moderation-action-object-action-types


AutoModerationActionTypes = Literal[1, 2, 3]


# https://discord.com/developers/docs/resources/auto-moderation#auto-moderation-action-object-action-metadata


class BlockmessageAutoModerationActionMetadataData(TypedDict):
    custom_message: NotRequired[str]


class SendAlertMessageAutoModerationActionMetadataData(TypedDict):
    channel_id: 'discord_typings.Snowflake'


class TimeoutAutoModerationActionMetadataData(TypedDict):
    duration_seconds: int


AutoModerationActionMetadataData = Union[
    BlockmessageAutoModerationActionMetadataData,
    SendAlertMessageAutoModerationActionMetadataData,
    TimeoutAutoModerationActionMetadataData,
]



from typing import Dict, Generic, List, Optional, TypeVar, Union

from typing_extensions import Literal, NotRequired, TypedDict

import discord_typings

__all__ = (
    'ApplicationCommandData',
    'ApplicationCommandTypes',
    'SubcommandOptionData',
    'SubcommandGroupOptionData',
    'ApplicationCommandOptionData',
    'ApplicationCommandOptionTypes',
    'CommandOptionChoiceData',
    'GuildApplicationCommandPermissionData',
    'ApplicationCommandPermissionsData',
    'ApplicationCommandPermissionTypes',
    'ApplicationCommandPayload',
)


_T = TypeVar('_T')

# https://discord.com/developers/docs/interactions/application-commands#application-command-object


class _ChatInputCommandData(TypedDict):
    id: 'discord_typings.Snowflake'
    type: NotRequired[Literal[1]]
    application_id: 'discord_typings.Snowflake'
    guild_id: NotRequired['discord_typings.Snowflake']
    name: str
    name_localizations: NotRequired[Optional[Dict['discord_typings.Locales', str]]]
    description: str
    description_localizations: NotRequired[Optional[Dict['discord_typings.Locales', str]]]
    options: List['discord_typings.ApplicationCommandOptionData']
    default_member_permissions: Optional[str]
    dm_permission: NotRequired[bool]
    nsfw: NotRequired[bool]
    integration_types: NotRequired[List['discord_typings.ApplicationIntegrationTypes']]
    contexts: NotRequired[List['discord_typings.InteractionContextTypes']]
    version: 'discord_typings.Snowflake'


class _ContextMenuCommandData(TypedDict):
    id: 'discord_typings.Snowflake'
    type: NotRequired[Literal[2, 3]]
    application_id: 'discord_typings.Snowflake'
    guild_id: NotRequired['discord_typings.Snowflake']
    name: str
    name_localizations: NotRequired[Optional[Dict['discord_typings.Locales', str]]]
    description: str
    description_localizations: NotRequired[Optional[Dict['discord_typings.Locales', str]]]
    default_member_permissions: Optional[str]
    dm_permission: NotRequired[bool]
    nsfw: NotRequired[bool]
    integration_types: NotRequired[List['discord_typings.ApplicationIntegrationTypes']]
    contexts: NotRequired[List['discord_typings.InteractionContextTypes']]
    version: 'discord_typings.Snowflake'


ApplicationCommandData = Union[_ChatInputCommandData, _ContextMenuCommandData]


# https://discord.com/developers/docs/interactions/application-commands#application-command-object-application-command-types


ApplicationCommandTypes = Literal[1, 2, 3]


# https://discord.com/developers/docs/interactions/application-commands#application-command-object-application-command-option-structure

# The variations and overloads of options are huge, but we want to allow
# type narrowing. As a result, this becomes a very large union.

# While Discord only documents a "full" ApplicationCommandOptionData,
# due to the nature of subcommand options those are also exposed


class SubcommandOptionData(TypedDict):
    type: Literal[1]
    name: str
    name_localizations: NotRequired[Optional[Dict['discord_typings.Locales', str]]]
    description: str
    description_localizations: NotRequired[Optional[Dict['discord_typings.Locales', str]]]
    options: NotRequired[List['discord_typings.ApplicationCommandOptionData']]


class SubcommandGroupOptionData(TypedDict):
    type: Literal[2]
    name: str
    name_localizations: NotRequired[Optional[Dict['discord_typings.Locales', str]]]
    description: str
    description_localizations: NotRequired[Optional[Dict['discord_typings.Locales', str]]]
    options: List['discord_typings.SubcommandOptionData']


class _ChoicesStringOptionData(TypedDict):
    type: Literal[3]
    name: str
    name_localizations: NotRequired[Optional[Dict['discord_typings.Locales', str]]]
    description: str
    description_localizations: NotRequired[Optional[Dict['discord_typings.Locales', str]]]
    required: NotRequired[bool]
    choices: NotRequired[List['discord_typings.CommandOptionChoiceData[str]']]


class _AutocompleteStringOptionData(TypedDict):
    type: Literal[3]
    name: str
    name_localizations: NotRequired[Optional[Dict['discord_typings.Locales', str]]]
    description: str
    description_localizations: NotRequired[Optional[Dict['discord_typings.Locales', str]]]
    required: NotRequired[bool]
    autocomplete: NotRequired[bool]


class _MinMaxStringOptionData(TypedDict):
    type: Literal[3]
    name: str
    name_localizations: NotRequired[Optional[Dict['discord_typings.Locales', str]]]
    description: str
    description_localizations: NotRequired[Optional[Dict['discord_typings.Locales', str]]]
    required: NotRequired[bool]
    min_length: NotRequired[int]
    max_length: NotRequired[int]


class _ChoicesIntegerCommandOptionData(TypedDict):
    type: Literal[4]
    name: str
    name_localizations: NotRequired[Optional[Dict['discord_typings.Locales', str]]]
    description: str
    description_localizations: NotRequired[Optional[Dict['discord_typings.Locales', str]]]
    required: NotRequired[bool]
    choices: NotRequired[List['discord_typings.CommandOptionChoiceData[int]']]


class _MinMaxIntegerCommandOptionData(TypedDict):
    type: Literal[4]
    name: str
    name_localizations: NotRequired[Optional[Dict['discord_typings.Locales', str]]]
    description: str
    description_localizations: NotRequired[Optional[Dict['discord_typings.Locales', str]]]
    required: NotRequired[bool]
    min_value: NotRequired[int]
    max_value: NotRequired[int]


class _AutocompleteIntegerOptionData(TypedDict):
    type: Literal[4]
    name: str
    name_localizations: NotRequired[Optional[Dict['discord_typings.Locales', str]]]
    description: str
    description_localizations: NotRequired[Optional[Dict['discord_typings.Locales', str]]]
    required: NotRequired[bool]
    autocomplete: NotRequired[bool]


class _BooleanOptionData(TypedDict):
    type: Literal[5]
    name: str
    name_localizations: NotRequired[Optional[Dict['discord_typings.Locales', str]]]
    description: str
    description_localizations: NotRequired[Optional[Dict['discord_typings.Locales', str]]]
    required: NotRequired[bool]


class _UserOptionData(TypedDict):
    type: Literal[6]
    name: str
    name_localizations: NotRequired[Optional[Dict['discord_typings.Locales', str]]]
    description: str
    description_localizations: NotRequired[Optional[Dict['discord_typings.Locales', str]]]
    required: NotRequired[bool]


class _ChannelOptionData(TypedDict):
    type: Literal[7]
    name: str
    name_localizations: NotRequired[Optional[Dict['discord_typings.Locales', str]]]
    description: str
    description_localizations: NotRequired[Optional[Dict['discord_typings.Locales', str]]]
    required: NotRequired[bool]
    channel_types: NotRequired[List['discord_typings.ChannelTypes']]


class _RoleOptionData(TypedDict):
    type: Literal[8]
    name: str
    name_localizations: NotRequired[Optional[Dict['discord_typings.Locales', str]]]
    description: str
    description_localizations: NotRequired[Optional[Dict['discord_typings.Locales', str]]]
    required: NotRequired[bool]


class _MentionableOptionData(TypedDict):
    type: Literal[9]
    name: str
    name_localizations: NotRequired[Optional[Dict['discord_typings.Locales', str]]]
    description: str
    description_localizations: NotRequired[Optional[Dict['discord_typings.Locales', str]]]
    required: NotRequired[bool]


class _ChoicesNumberCommandOptionData(TypedDict):
    type: Literal[10]
    name: str
    name_localizations: NotRequired[Optional[Dict['discord_typings.Locales', str]]]
    description: str
    description_localizations: NotRequired[Optional[Dict['discord_typings.Locales', str]]]
    required: NotRequired[bool]
    choices: NotRequired[List['discord_typings.CommandOptionChoiceData[Union[int, float]]']]


class _MinMaxNumberCommandOptionData(TypedDict):
    type: Literal[10]
    name: str
    name_localizations: NotRequired[Optional[Dict['discord_typings.Locales', str]]]
    description: str
    description_localizations: NotRequired[Optional[Dict['discord_typings.Locales', str]]]
    required: NotRequired[bool]
    min_value: NotRequired[Union[int, float]]
    max_value: NotRequired[Union[int, float]]


class _AutocompleteNumberOptionData(TypedDict):
    type: Literal[10]
    name: str
    name_localizations: NotRequired[Optional[Dict['discord_typings.Locales', str]]]
    description: str
    description_localizations: NotRequired[Optional[Dict['discord_typings.Locales', str]]]
    required: NotRequired[bool]
    autocomplete: NotRequired[bool]


class _AttachmentOptionData(TypedDict):
    type: Literal[11]
    name: str
    name_localizations: NotRequired[Optional[Dict['discord_typings.Locales', str]]]
    description: str
    description_localizations: NotRequired[Optional[Dict['discord_typings.Locales', str]]]
    required: NotRequired[bool]


ApplicationCommandOptionData = Union[
    SubcommandOptionData,
    SubcommandGroupOptionData,
    _ChoicesStringOptionData,
    _AutocompleteStringOptionData,
    _MinMaxStringOptionData,
    _ChoicesIntegerCommandOptionData,
    _MinMaxIntegerCommandOptionData,
    _AutocompleteIntegerOptionData,
    _BooleanOptionData,
    _UserOptionData,
    _ChannelOptionData,
    _RoleOptionData,
    _MentionableOptionData,
    _ChoicesNumberCommandOptionData,
    _MinMaxNumberCommandOptionData,
    _AutocompleteNumberOptionData,
    _AttachmentOptionData,
]


# https://discord.com/developers/docs/interactions/application-commands#application-command-object-application-command-option-type


ApplicationCommandOptionTypes = Literal[1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11]


# https://discord.com/developers/docs/interactions/application-commands#application-command-object-application-command-option-choice-structure


class CommandOptionChoiceData(TypedDict, Generic[_T]):
    name: str
    name_localizations: NotRequired[Optional[Dict['discord_typings.Locales', str]]]
    value: _T


# https://discord.com/developers/docs/interactions/application-commands#application-command-permissions-object


class GuildApplicationCommandPermissionData(TypedDict):
    id: 'discord_typings.Snowflake'
    application_id: 'discord_typings.Snowflake'
    guild_id: 'discord_typings.Snowflake'
    permissions: List['discord_typings.ApplicationCommandPermissionsData']


# https://discord.com/developers/docs/interactions/application-commands#application-command-permissions-object-application-command-permissions-structure


class ApplicationCommandPermissionsData(TypedDict):
    id: 'discord_typings.Snowflake'
    type: 'discord_typings.ApplicationCommandPermissionTypes'
    permission: bool


# https://discord.com/developers/docs/interactions/application-commands#application-command-permissions-object-application-command-permission-type


ApplicationCommandPermissionTypes = Literal[1, 2, 3]


# https://discord.com/developers/docs/interactions/application-commands#create-global-application-command-json-params


class ApplicationCommandPayload(TypedDict):
    name: str
    name_localizations: NotRequired[Optional[Dict['discord_typings.Locales', str]]]
    description: NotRequired[str]
    description_localizations: NotRequired[Optional[Dict['discord_typings.Locales', str]]]
    options: NotRequired[List['discord_typings.ApplicationCommandOptionData']]
    default_member_permissions: NotRequired[Optional[str]]
    dm_permission: NotRequired[Optional[bool]]
    integration_types: NotRequired[List['discord_typings.ApplicationIntegrationTypes']]
    contexts: NotRequired[List['discord_typings.InteractionContextTypes']]
    type: NotRequired['discord_typings.ApplicationCommandTypes']
    nsfw: NotRequired[bool]



from __future__ import annotations

from typing import Dict, Generic, List, TypeVar, Union

from typing_extensions import Literal, NotRequired, TypedDict

import discord_typings

__all__ = (
    'PingInteractionData',
    'ApplicationCommandInteractionData',
    'ComponentInteractionData',
    'AutocompleteInteractionData',
    'ModalInteractionData',
    'InteractionData',
    'InteractionType',
    'InteractionTypes',
    'InteractionContextTypes',
    'ApplicationCommandInteractionDataData',
    'ComponentInteractionDataData',
    'ModalSubmitInteractionDataData',
    'InteractionDataData',
    'ResolvedInteractionDataData',
    'ApplicationCommandOptionInteractionData',
    'MessageInteractionData',
    'InteractionResponseData',
    'InteractionCallbackTypes',
    'InteractionMessageCallbackData',
    'InteractionAutocompleteCallbackData',
    'InteractionModalCallbackData',
    'InteractionCallbackData',
)


_T = TypeVar('_T')


# https://discord.com/developers/docs/interactions/receiving-and-responding#interaction-object


class PingInteractionData(TypedDict):
    id: 'discord_typings.Snowflake'
    application_id: 'discord_typings.Snowflake'
    type: Literal[1]
    user: 'discord_typings.UserData'
    version: int


class _GuildInteractionData(TypedDict):
    id: 'discord_typings.Snowflake'
    application_id: 'discord_typings.Snowflake'
    guild_id: 'discord_typings.Snowflake'
    channel: 'discord_typings.PartialChannelData'
    channel_id: 'discord_typings.Snowflake'
    member: 'discord_typings.GuildMemberData'
    token: str
    version: int
    app_permissions: str
    locale: 'discord_typings.Locales'
    guild_locale: 'discord_typings.Locales'
    entitlements: NotRequired[List['discord_typings.EntitlementData']]
    authorizing_integration_owners: Dict[
        'discord_typings.ApplicationIntegrationTypes',
        'discord_typings.Snowflake'
    ]
    context: Literal[0]


class _ApplicationCommandGuildInteractionData(_GuildInteractionData):
    type: Literal[2]
    data: 'discord_typings.ApplicationCommandInteractionDataData'


class _ComponentGuildInteractionData(_GuildInteractionData):
    type: Literal[3]
    data: 'discord_typings.ComponentInteractionDataData'
    message: 'discord_typings.MessageData'


class _AutocompleteGuildInteractionData(_GuildInteractionData):
    type: Literal[4]
    data: 'discord_typings.ApplicationCommandInteractionDataData'


class _ModalGuildInteractionData(_GuildInteractionData):
    type: Literal[5]
    data: 'discord_typings.ModalSubmitInteractionDataData'


class _ChannelInteractionData(TypedDict):
    id: 'discord_typings.Snowflake'
    application_id: 'discord_typings.Snowflake'
    channel: 'discord_typings.PartialChannelData'
    channel_id: 'discord_typings.Snowflake'
    user: 'discord_typings.UserData'
    token: str
    version: int
    locale: 'discord_typings.Locales'
    entitlements: NotRequired[List['discord_typings.EntitlementData']]
    authorizing_integration_owners: Dict[
        'discord_typings.ApplicationIntegrationTypes',
        'discord_typings.Snowflake'
    ]
    context: Literal[1, 2]


class _ApplicationCommandChannelInteractionData(_ChannelInteractionData):
    type: Literal[2]
    data: 'discord_typings.ApplicationCommandInteractionDataData'


class _ComponentChannelInteractionData(_ChannelInteractionData):
    type: Literal[3]
    data: 'discord_typings.ComponentInteractionDataData'
    message: 'discord_typings.MessageData'


class _AutocompleteChannelInteractionData(_ChannelInteractionData):
    type: Literal[4]
    data: 'discord_typings.ApplicationCommandInteractionDataData'


class _ModalChannelInteractionData(_ChannelInteractionData):
    type: Literal[5]
    data: 'discord_typings.ModalSubmitInteractionDataData'


ApplicationCommandInteractionData = Union[
    _ApplicationCommandGuildInteractionData, _ApplicationCommandChannelInteractionData,
]


ComponentInteractionData = Union[
    _ComponentGuildInteractionData, _ComponentChannelInteractionData
]


AutocompleteInteractionData = Union[
    _AutocompleteGuildInteractionData, _AutocompleteChannelInteractionData
]


ModalInteractionData = Union[
    _ModalGuildInteractionData, _ModalChannelInteractionData
]


InteractionData = Union[
    ApplicationCommandInteractionData, ComponentInteractionData, AutocompleteInteractionData,
    ModalInteractionData,
]


# https://discord.com/developers/docs/interactions/receiving-and-responding#interaction-object-interaction-type


InteractionType = Literal[1, 2, 3, 4, 5]
InteractionTypes = InteractionType  # Alias


# https://discord.com/developers/docs/interactions/receiving-and-responding#interaction-object-interaction-context-types


InteractionContextTypes = Literal[0, 1, 2]


# https://discord.com/developers/docs/interactions/receiving-and-responding#interaction-object-application-command-data-structure


class _ChatInputCommandInteractionDataData(TypedDict):
    id: 'discord_typings.Snowflake'
    name: str
    type: Literal[1]
    resolved: NotRequired[ResolvedInteractionDataData]
    options: NotRequired[List[ApplicationCommandOptionInteractionData]]
    guild_id: NotRequired['discord_typings.Snowflake']


class _ContextMenuInteractionDataData(TypedDict):
    id: 'discord_typings.Snowflake'
    name: str
    type: Literal[2, 3]
    resolved: NotRequired[ResolvedInteractionDataData]
    guild_id: NotRequired['discord_typings.Snowflake']
    target_id: 'discord_typings.Snowflake'


ApplicationCommandInteractionDataData = Union[
    _ChatInputCommandInteractionDataData, _ContextMenuInteractionDataData
]


# https://discord.com/developers/docs/interactions/receiving-and-responding#interaction-object-message-component-data-structure


class _ButtonComponentInteractionDataData(TypedDict):
    custom_id: str
    component_type: Literal[2]


class _SelectMenuComponentInteractionDataData(TypedDict):
    custom_id: str
    component_type: Literal[3, 5, 6, 7, 8]
    values: NotRequired[List['discord_typings.SelectMenuOptionData']]
    resolved: NotRequired[ResolvedInteractionDataData]


ComponentInteractionDataData = Union[
    _ButtonComponentInteractionDataData, _SelectMenuComponentInteractionDataData
]


# https://discord.com/developers/docs/interactions/receiving-and-responding#interaction-object-modal-submit-data-structure


class ModalSubmitInteractionDataData(TypedDict):
    custom_id: str
    component_type: Literal[4]
    components: List['discord_typings.ActionRowData']


InteractionDataData = Union[
    ApplicationCommandInteractionDataData, ComponentInteractionDataData,
    ModalSubmitInteractionDataData
]


# https://discord.com/developers/docs/interactions/receiving-and-responding#interaction-object-resolved-data-structure


class ResolvedInteractionDataData(TypedDict):
    users: NotRequired[
        Dict['discord_typings.Snowflake', 'discord_typings.UserData']
    ]
    members: NotRequired[
        Dict['discord_typings.Snowflake', 'discord_typings.GuildMemberData']
    ]
    roles: NotRequired[
        Dict['discord_typings.Snowflake', 'discord_typings.RoleData']
    ]
    channels: NotRequired[
        Dict['discord_typings.Snowflake', 'discord_typings.PartialChannelData']
    ]
    messages: NotRequired[
        Dict['discord_typings.Snowflake', 'discord_typings.MessageData']
    ]
    attachments: NotRequired[
        Dict['discord_typings.Snowflake', 'discord_typings.AttachmentData']
    ]


# https://discord.com/developers/docs/interactions/receiving-and-responding#interaction-object-application-command-interaction-data-option-structure


class _SubcommandOptionInteractionData(TypedDict):
    name: str
    type: Literal[1]
    options: List['discord_typings.ApplicationCommandOptionInteractionData']


class _SubcommandGroupOptionInteractionData(TypedDict):
    name: str
    type: Literal[2]
    options: List['discord_typings.ApplicationCommandOptionInteractionData']


class _StringOptionInteractionData(TypedDict):
    name: str
    type: Literal[3]
    value: str
    focused: NotRequired[bool]


class _IntegerOptionInteractionData(TypedDict):
    name: str
    type: Literal[4]
    value: int
    focused: NotRequired[bool]


class _BooleanOptionInteractionData(TypedDict):
    name: str
    type: Literal[5]
    value: bool


class _UserOptionInteractionData(TypedDict):
    name: str
    type: Literal[6]
    value: 'discord_typings.Snowflake'


class _ChannelOptionInteractionData(TypedDict):
    name: str
    type: Literal[7]
    value: 'discord_typings.Snowflake'


class _RoleOptionInteractionData(TypedDict):
    name: str
    type: Literal[8]
    value: 'discord_typings.Snowflake'


class _MentionableInteractionData(TypedDict):
    name: str
    type: Literal[9]
    value: 'discord_typings.Snowflake'


class _NumberInteractionData(TypedDict):
    name: str
    type: Literal[10]
    value: Union[int, float]
    focused: NotRequired[bool]


class _AttachmentInteractionData(TypedDict):
    name: str
    type: Literal[11]
    value: str


ApplicationCommandOptionInteractionData = Union[
    _SubcommandOptionInteractionData, _SubcommandGroupOptionInteractionData,
    _StringOptionInteractionData, _IntegerOptionInteractionData, _BooleanOptionInteractionData,
    _UserOptionInteractionData, _ChannelOptionInteractionData, _RoleOptionInteractionData,
    _MentionableInteractionData, _NumberInteractionData, _AttachmentInteractionData,
]


# https://discord.com/developers/docs/interactions/receiving-and-responding#message-interaction-object-message-interaction-structure


class MessageInteractionData(TypedDict):
    id: 'discord_typings.Snowflake'
    type: Literal[2, 3, 4]
    name: str
    user: 'discord_typings.UserData'
    member: NotRequired['discord_typings.GuildMemberData']


# https://discord.com/developers/docs/interactions/receiving-and-responding#interaction-response-object-interaction-response-structure


class _InteractionNodataResponseData(TypedDict):
    type: Literal[1, 5, 6, 10]


class _InteractionMessageResponseData(TypedDict):
    type: Literal[4, 7]
    data: 'discord_typings.InteractionMessageCallbackData'


class _InteractionAutocompleteResponseData(TypedDict):
    type: Literal[8]
    data: 'discord_typings.InteractionAutocompleteCallbackData[Union[str, int, float]]'


class _InteractionModalResponseData(TypedDict):
    type: Literal[9]
    data: 'discord_typings.InteractionModalCallbackData'


InteractionResponseData = Union[
    _InteractionNodataResponseData, _InteractionMessageResponseData,
    _InteractionAutocompleteResponseData, _InteractionModalResponseData,
]


# https://discord.com/developers/docs/interactions/receiving-and-responding#interaction-response-object-interaction-callback-type


InteractionCallbackTypes = Literal[1, 4, 5, 6, 7, 8, 9, 10]


# https://discord.com/developers/docs/interactions/receiving-and-responding#interaction-response-object-interaction-callback-data-structure


class InteractionMessageCallbackData(TypedDict):
    tts: NotRequired[bool]
    content: NotRequired[str]
    embeds: NotRequired[List['discord_typings.EmbedData']]
    allowed_mentions: NotRequired['discord_typings.AllowedMentionsData']
    flags: NotRequired[int]
    components: NotRequired[List['discord_typings.ComponentData']]
    attachments: NotRequired[List['discord_typings.PartialAttachmentData']]


class InteractionAutocompleteCallbackData(TypedDict, Generic[_T]):
    choices: List['discord_typings.CommandOptionChoiceData[_T]']


class InteractionModalCallbackData(TypedDict):
    custom_id: str
    title: str
    components: List['discord_typings.ComponentData']


InteractionCallbackData = Union[
    InteractionMessageCallbackData,
    InteractionAutocompleteCallbackData[Union[str, int, float]],
    InteractionModalCallbackData,
]



"""Python typings of all payloads that Discord sends as `TypedDict`s."""

# This file is not sorted by isort, but try to order the imports by
# name. The reason for this is because of circular imports in
# subclassing forcing imports to be in a certain order

# User needs to be imported before channel, due to UserMentionData subclass
from ._resources._user import *

from ._interactions._commands import *
from ._interactions._components import *
from ._interactions._receiving import *
from ._monetization._entitlements import *
from ._oauth import *
from ._reference import *
from ._resources._application import *
from ._resources._audit_log import *
from ._resources._auto_moderation import *
from ._resources._channel import *
from ._resources._emoji import *
from ._resources._guild import *
from ._resources._guild_scheduled_events import *
from ._resources._guild_template import *
from ._resources._invite import *
from ._resources._role_connection_metadata import *
from ._resources._stage_instance import *
from ._resources._sticker import *
from ._resources._voice import *
from ._resources._webhook import *

# Gateway imports several other things, so it is imported last
from ._gateway import *



from typing import Dict, Optional

from typing_extensions import Literal, NotRequired, TypedDict

import discord_typings

__all__ = (
    'ApplicationRoleConnectionMetadataData',
    'ApplicationRoleConnectionMetadataTypes',
)


# https://discord.com/developers/docs/resources/application-role-connection-metadata#application-role-connection-metadata-object


class ApplicationRoleConnectionMetadataData(TypedDict):
    type: 'discord_typings.ApplicationRoleConnectionMetadataTypes'
    key: str
    name: str
    name_localizations: NotRequired[Optional[Dict['discord_typings.Locales', str]]]
    description: str
    description_localizations: NotRequired[Optional[Dict['discord_typings.Locales', str]]]


# https://discord.com/developers/docs/resources/application-role-connection-metadata#application-role-connection-metadata-object-application-role-connection-metadata-type


ApplicationRoleConnectionMetadataTypes = Literal[1, 2, 3, 4, 5, 6, 7, 8]



from typing import Dict, List, Optional

from typing_extensions import Literal, NotRequired, TypedDict

import discord_typings

__all__ = (
    'ApplicationData',
    'ApplicationIntegrationTypes',
    'ApplicationIntegrationTypeConfigurationData',
    'InstallParams',
    'TeamMemberRoleTypes',
    'TeamData',
    'TeamMemberData',
)


# https://discord.com/developers/docs/resources/application#application-object-application-structure


class ApplicationData(TypedDict):
    id: 'discord_typings.Snowflake'
    name: str
    icon: Optional[str]
    description: str
    rpc_origins: NotRequired[List[str]]
    bot_public: bool
    bot_require_code_grant: bool
    terms_of_service_url: NotRequired[str]
    privacy_policy_url: NotRequired[str]
    owner: NotRequired['discord_typings.UserData']
    verify_key: str
    team: Optional['discord_typings.TeamData']
    guild_id: NotRequired['discord_typings.Snowflake']
    guild: NotRequired['discord_typings.PartialGuildData']
    primary_sku_id: NotRequired['discord_typings.Snowflake']
    slug: NotRequired[str]
    cover_image: NotRequired[str]
    flags: NotRequired[int]
    approximate_guild_count: NotRequired[int]
    redirect_urls: NotRequired[List[str]]
    interactions_endpoint_url: NotRequired[str]
    role_connections_verification_url: NotRequired[str]
    tags: NotRequired[List[str]]
    install_params: NotRequired['discord_typings.InstallParams']
    integration_types_config: Dict[
        'discord_typings.ApplicationIntegrationTypes',
        'discord_typings.ApplicationIntegrationTypeConfigurationData'
    ]
    custom_install_url: NotRequired[str]


# https://discord.com/developers/docs/resources/application#application-object-application-integration-types


ApplicationIntegrationTypes = Literal[
    0,
    1,
]


# https://discord.com/developers/docs/resources/application#application-object-application-integration-type-configuration-object


class ApplicationIntegrationTypeConfigurationData(TypedDict):
    oauth2_install_params: 'discord_typings.InstallParams'


# https://discord.com/developers/docs/resources/application#install-params-object-install-params-structure


class InstallParams(TypedDict):
    scopes: List['discord_typings.OAuth2Scopes']
    permissions: str


# https://discord.com/developers/docs/topics/teams#team-member-roles


TeamMemberRoleTypes = Literal['', 'admin', 'developer', 'read_only']


# https://discord.com/developers/docs/topics/teams#data-models-team-object


class TeamData(TypedDict):
    icon: Optional[str]
    id: 'discord_typings.Snowflake'
    members: List['discord_typings.TeamMemberData']
    name: str
    owner_user_id: 'discord_typings.Snowflake'


# https://discord.com/developers/docs/topics/teams#data-models-team-member-object


class TeamMemberData(TypedDict):
    membership_state: int
    team_id: 'discord_typings.Snowflake'
    user: 'discord_typings.UserData'
    role: str
